/* Clenshaw-Curtis integration over the deltoid using precomputed integrals of Chebyshev polynomials.
 * Input:
 *	Values : Values of the function on the Chebyshev extremal points.
 *	IntT : Table of integrals of Chebyshev polynomials from IntLookup_deltoid().
 * Output:
 *	The integral over the inscribed triangle.
 *
 * Brett Ryland, April 2009
 */

#include "Chebyshev.hpp"
#include <fftw3.h>

double CCInt(DMatrix Values, DMatrix IntT)
{
  int N;
  double I,Ndual;
  IVector Niso;
  IMatrix g;
  IArray G;
  CMatrix Coefficients;
  fftw_plan plan;

  N=Values.size1();
  if( Values.size2()!=(unsigned)N ) {
	std::cerr << "Input array is the wrong size" << std::endl;
	return 0.;
  }

  Coefficients.resize(N,N/2+1);
  // perform a FFT on symmetrised Values to obtain the Chebyshev coefficients
  // plan the 2D FFT, maybe this could be improved by only planning it once?
  plan = fftw_plan_dft_r2c_2d(N, N, &(Values.data()[0]), reinterpret_cast<fftw_complex*>(&(Coefficients.data()[0])), FFTW_ESTIMATE);
  fftw_execute(plan);
  Coefficients/=N*N;

  // obtain the orbit information of the multivariate Chebyshev polynomials in the fundamental domain.
  g = dualfudo(N);
  G = Gdual();
  Ndual = G.size();
  Niso = orbit_info(G,g,N);

  // perform the integration
  // Note: the Coefficients are not real but the imaginary part vanishes due to the symmetries of the integral,
  // so we only take the real part, i.e., Coefficients[0]
  I=0.;
  for( unsigned int i=0; i<g.size2(); ++i )
	I += real(Coefficients(g(0,i),g(1,i)))*IntT(g(0,i),g(1,i))*Ndual/Niso(i);

  // clean up
  fftw_destroy_plan(plan);
  return I;
}
