/* Create a table of Integration values over the standard triangle for all multivariate Chebyshev
 * polynomials within the dual fundamental domain for a given value of the downscaling lattice.
 *
 * Input:
 *  N : Downscaling factor.
 *
 * Output:
 *  The table of integrals over the standard triangle of each Chebyshev polynomial in the dual fundamental domain.
 *
 * Brett Ryland, April 2009
 */

#include "Chebyshev.hpp"
#include <boost/numeric/ublas/matrix_proxy.hpp>

double evalInt(int k1, int k2)
{
  // k1~=0, k2~=0, k1+k2~=0
  /* actual function is
   * 1./k1/k2*(exp(i*k1*M_PI/3.)*(exp(i*k2*2.*M_PI/3.)-exp(i*k2*M_PI/3))-exp(i*k1*M_PI)*(exp(i*k2*M_PI)-exp(i*k2*2*M_PI/3)))+1/k1/(k2+k1)*(exp(2/3*i*M_PI*k1)*(exp(i*M_PI*k2)+exp(1/3*i*M_PI*k2))-exp(2/3*i*M_PI*k2)*(exp(i*k1*M_PI)+exp(1/3*i*k1*M_PI)));
   * but real parts cancel due to symmetries of W. I.e., sum_{j}(evalInt(W(j)*k)) is purely imaginary.
   * So, we only return the imaginary component as a real double
   */
  return 1./k1/k2*(cos(k1/3.*M_PI)*(sin(k2*2./3.*M_PI)-sin(k2/3.*M_PI))+sin(k1/3.*M_PI)*(cos(k2*2./3.*M_PI)-cos(k2/3.*M_PI))-cos(k1*M_PI)*(sin(k2*M_PI)-sin(k2*2./3.*M_PI))-sin(k1*M_PI)*(cos(k2*M_PI)-cos(k2*2./3.*M_PI)))+1./k1/(k2+k1)*(cos(k1*2./3.*M_PI)*(sin(k2*M_PI)+sin(k2/3.*M_PI))+sin(k1*2./3.*M_PI)*(cos(k2*M_PI)+cos(k2/3.*M_PI))-cos(k2*2./3.*M_PI)*(sin(k1*M_PI)+sin(k1/3.*M_PI))-sin(k2*2./3.*M_PI)*(cos(k1*M_PI)+cos(k1/3.*M_PI)));
}

DMatrix IntLookup_triangle(int N)
{
  using namespace boost::numeric::ublas;
  IVector k(2);
  IMatrix g,l(2,6),K(3,36);
  IArray W;
  DMatrix IntT;

  // figure out which Chebyshev polynomials we need
  g=dualfudo(N);

  // construct the Weyl group and Jacobian determinant modifiers to k
  W=Weyl('A',2);
  l(0,0)=1;l(0,1)=-1;l(0,2)=1;l(0,3)=-1;l(0,4)=-2;l(0,5)=2;
  l(1,0)=1;l(1,1)=-1;l(1,2)=-2;l(1,3)=2;l(1,4)=1;l(1,5)=-1;

  IntT.resize(norm_inf(row(g,0))+1,norm_inf(row(g,1))+1);
  IntT.clear();
  for( unsigned int n=0; n<g.size2(); ++n) {
	k=column(g,n);
	if( (k(0)-k(1))%3!=0 )
	  continue; // integral over the triangle is zero unless mod(k(1)-k(2),3) == 0

	// construct the points to evaluate the integral at
	for( unsigned int j1=0; j1<W.size(); ++j1 )
	  for( unsigned int j2=0; j2<W.size(); ++j2 ) {
		matrix_vector_slice<IMatrix> mvs(K, slice(0,1,2), slice(j1*W.size()+j2,0,2));
		mvs=prod(W(j1),k)+column(l,j2);
		K(2,j1*6+j2)=(j2%2==0)?(1):(-1);
	  }
	// sum over the integrals
	for( unsigned int j=0;j<W.size()*W.size(); ++j) {
	  // the first four possibilities always occur in +- pairs which cancel, so ignore them.
	  if( K(0,j)==0 && K(1,j)==0 )
		continue;
	  if( K(0,j)==0 ) // K(1,j)~=0 if we got to here
		continue;
	  if( K(1,j)==0 ) // K(0,j)~=0 if we got to here
		continue;
	  if( K(0,j)+K(1,j)==0 ) // K(0,j)~=0, K(1,j)~=0 if we got to here
		continue;
	  IntT(g(0,n),g(1,n))-=K(2,j)*evalInt(K(0,j),K(1,j));
	}
  }
  // 1/2 from |J_x(z)|/i, -1/9 from |J_z(theta)|, 1/6 from 1/|W| and - from clockwise to anti-clockwise orientation of triangle in x and theta domains
  IntT=IntT/108.;
  return IntT;
}
