/* Construct the integrand of a function on a nonlinear triangle mapped back to the equilateral triangle.
 *
 * Input:
 *	f (N,N) = function on the nonlinear triangle
 *	J (2,2,N,N) or (2,2) = Jacobian of the nonlinear mapping at each point in the domain,
 *		                   or a single Jacobian for a linear mapping.
 *
 * Output:
 *	F (N,N) = integrand of the function on the equilateral triangle
 *
 * Brett Ryland, August 2009
 */

#include "Chebyshev.hpp"

DMatrix TriangleMap(DMatrix f, JArray J)
{
  int N;
  DMatrix F;
  DArray gradf;

  N=f.size1();
  gradf=Gradient(f);
  // gradf is multiplied by the inverse transpose of the Jacobian at each point
  for( int i=0; i<N; ++i )
	for( int j=0; j<N; ++j ) {
	  double det=J(i,j)(0,0)*J(i,j)(1,1)-J(i,j)(0,1)*J(i,j)(1,0);
	  double temp=(J(i,j)(1,1)*gradf(0)(i,j)-J(i,j)(1,0)*gradf(1)(i,j))/det;
	  gradf(1)(i,j)=(-J(i,j)(0,1)*gradf(0)(i,j)+J(i,j)(0,0)*gradf(1)(i,j))/det;
	  gradf(0)(i,j)=temp;
	}

  F.resize(N,N);
  // surface area function (adjust this for the desired integrand)
  for( int i=0; i<N; ++i )
	for( int j=0; j<N; ++j )
	  F(i,j)=sqrt(1.+gradf(0)(i,j)*gradf(0)(i,j)+gradf(1)(i,j)*gradf(1)(i,j));

  // phi is a nonlinear mapping so we multiply F by abs(det(J)) at each point
  for( int i=0; i<N; ++i )
	for( int j=0; j<N; ++j )
	  F(i,j)*=fabs(J(i,j)(0,0)*J(i,j)(1,1)-J(i,j)(0,1)*J(i,j)(1,0));

  return F;
}

DMatrix TriangleMap(DMatrix f, DMatrix J)
{
  int N;
  double det;
  DMatrix F;
  DArray gradf;

  N=f.size1();
  gradf=Gradient(f);
  // gradf is multiplied by the inverse transpose of the Jacobian
  det=J(0,0)*J(1,1)-J(0,1)*J(1,0);
  for( int i=0; i<N; ++i )
	for( int j=0; j<N; ++j ) {
	  double temp=(J(1,1)*gradf(0)(i,j)-J(1,0)*gradf(1)(i,j))/det;
	  gradf(1)(i,j)=(-J(0,1)*gradf(0)(i,j)+J(0,0)*gradf(1)(i,j))/det;
	  gradf(0)(i,j)=temp;
	}

  F.resize(N,N);
  // surface area function (adjust this for the desired integrand)
  for( int i=0; i<N; ++i )
	for( int j=0; j<N; ++j )
	  F(i,j)=sqrt(1.+gradf(0)(i,j)*gradf(0)(i,j)+gradf(1)(i,j)*gradf(1)(i,j));

  // phi is a linear mapping and abs(det(J)) is a constant
  F*=fabs(det);

  return F;
}
