/* Create a table of Integration values over the deltoid for all multivariate Chebyshev polynomials
 * within the dual fundamental domain for a given value of the downscaling lattice.
 *
 * Input:
 *	N : Downscaling factor.
 *
 * Output:
 *	The table of integrals over the deltoid of each Chebyshev polynomial in the dual fundamental domain.
 *
 * Brett Ryland, April 2009
 */

#include "Chebyshev.hpp"
#include <boost/numeric/ublas/matrix_proxy.hpp>

DMatrix IntLookup_deltoid(int N)
{
  using namespace boost::numeric::ublas;
  IVector k(2);
  IMatrix g,l(2,6),K(3,36);
  IArray W;
  DMatrix IntT;

  // figure out which Chebyshev polynomials we need
  g=dualfudo(N);

  // construct the Weyl group and Jacobian determinant modifiers to k
  W=Weyl('A',2);
  l(0,0)=1;l(0,1)=-1;l(0,2)=1;l(0,3)=-1;l(0,4)=-2;l(0,5)=2;
  l(1,0)=1;l(1,1)=-1;l(1,2)=-2;l(1,3)=2;l(1,4)=1;l(1,5)=-1;

  IntT.resize(norm_inf(row(g,0))+1,norm_inf(row(g,1))+1);
  IntT.clear();
  for( unsigned int n=0; n<g.size2(); ++n) {
	k=column(g,n);
	if( (k(0)!=k(1)) && (abs(k(0)-k(1))!=3) )
	  continue; // integral over the deltoid is zero unless k1==k2 or abs(k(0)-k(1)) == 3

	// construct the points to evaluate the integral at
	for( unsigned int j1=0; j1<W.size(); ++j1 )
	  for( unsigned int j2=0; j2<W.size(); ++j2 ) {
		matrix_vector_slice<IMatrix> mvs(K, slice(0,1,2), slice(j1*W.size()+j2,0,2));
		mvs=prod(W(j1),k)+column(l,j2);
		K(2,j1*6+j2)=(j2%2==0)?(1):(-1);
	  }
	// sum over the integrals
	for( unsigned int j=0;j<W.size()*W.size(); ++j) {
	  if( (K(0,j)==0) && (K(1,j)==0) ) {
		// cancels due to symmetries
		continue;
	  }
	  if( K(0,j)==0 ) {
		// cancels due to symmetries
		continue;
	  }
	  if( K(1,j)+K(0,j)/2==0 ) {
		IntT(g(0,n),g(1,n))-=K(2,j)*4.*M_PI/3./K(0,j);
		continue;
	  }
	  if( K(1,j)+2*K(0,j)==0 ) {
		IntT(g(0,n),g(1,n))-=K(2,j)*-2.*M_PI/3./K(0,j);
		continue;
	  }
	  if( K(1,j)-K(0,j)==0 ) {
		IntT(g(0,n),g(1,n))-=K(2,j)*-2.*M_PI/3./K(0,j);
		continue;
	  }
	}
  }
  // 1/2 from |J_x(z)|/i, -1/9 from |J_z(theta)|, 1/6 from 1/|W| and - from clockwise to anti-clockwise orientation of triangle in x and theta domains
  IntT=IntT/108.;
  return IntT;
}
