/* Interpolate a function at several points, based on the function values at the Chebyshev extremal points.
 *
 * Input:
 *	Points : Coordinates of the points to be interpolated.
 *	Values : Values of the function on the Chebyshev extremal points.
 * 	C : Triangle corners as a 2x3 matrix.
 *
 * Output:
 *	The interpolated values.
 *
 * Brett Ryland, April 2009
 */

#include "Chebyshev.hpp"
#include <fftw3.h>

DVector Interp(DMatrix Points, DMatrix Values)
{
  DMatrix C(2,3);
  C(0,0)=-1./3.; C(0,1)=C(0,0); C(0,2)=2./3.;
  C(1,0)=-1./sqrt(3.); C(1,1)=-C(1,0); C(1,2)=0.;
  return Interp(Points,Values,C);
}

DVector Interp(DMatrix Points, DMatrix Values, DMatrix C)
{
  int n,N,Ndual;
  IVector Niso;
  DVector I;
  CVector z;
  IMatrix g;
  DMatrix J(2,2);
  CMatrix a,T;
  IArray G;
  fftw_plan plan;
  Map map;

  if( Points.size1()!=2 )
	Points=trans(Points);
  if( Points.size1()!=2 ) {
	std::cerr << "Input variable \"Points\" must be a 2xn or nx2 array." << std::endl;
	return I;
  }
  n=Points.size2(); // get the number of points to evaluate

  if( n==0 )
	return I;
  N=Values.size1(); // get the downscaling

  a.resize(N,N/2+1);
  // perform a FFT on symmetrised Values to obtain the Chebyshev coefficients
  // plan the 2D FFT, maybe this could be improved by only planning it once?
  plan = fftw_plan_dft_r2c_2d(N, N, &(Values.data()[0]), reinterpret_cast<fftw_complex*>(&(a.data()[0])), FFTW_ESTIMATE);
  fftw_execute(plan);
  a/=N*N;
  // clean up
  fftw_destroy_plan(plan);

  map=ArbitraryTriangleToStandardTriangle(C);
  J(0,0)=map.T(1,1);
  J(0,1)=-map.T(0,1);
  J(1,0)=-map.T(1,0);
  J(1,1)=map.T(0,0);
  J/=map.T(0,0)*map.T(1,1)-map.T(1,0)*map.T(0,1); // J=inv(map.T)
  z.resize(n);
  for( int i=0; i<n; ++i )
	z(i)=std::complex<double>(J(0,0)*(Points(0,i)-map.c(0))+J(0,1)*(Points(1,i)-map.c(1)),J(1,0)*(Points(0,i)-map.c(0))+J(1,1)*(Points(1,i)-map.c(1))); // obtain interpolation coordinates w.r.t. deltoid
  T = T_A2(z,N); // evaluate the Chebyshev polynomials at the interpolation points

  // obtain the orbit information of the multivariate Chebyshev polynomials in the fundamental domain.
  g = dualfudo(N);
  G = Gdual();
  Ndual = G.size();
  Niso = orbit_info(G,g,N);

  // perform the interpolation, discarding imaginary part which vanishes due to symmetries
  I.resize(n);
  for( int i=0; i<n; ++i )
	for( unsigned int j=0; j<g.size2(); ++j )
	  I(i) += real(a(g(0,j),g(1,j))*T(i,j))*Ndual/Niso(j);
  return I;
}
