/* Calculate the surface area of a triangular surface patch using Clenshaw-Curtis
 * quadrature on the derivative of a function on the deltoid.
 *
 * Input:
 *	Values : Values of the function on the Chebyshev extremal points.
 *	IntT : Table of integrals of Chebyshev polynomials from IntLookup_deltoid(N).
 *
 * Output:
 *	The surface area of the function on the deltoid.
 *
 * Brett Ryland, April 2009
 */

#include "Chebyshev.hpp"
#include <cmath>

double Surface_Area(DMatrix Values, DMatrix IntT)
{
  int N;
  DMatrix Integrand;
  DArray Derivative;

  N=Values.size1();

  // Calculate the gradient of Function at the Chebyshev extremal points.
  Derivative = Gradient(Values);

  // Construct the function to be integrated.
  // Area element dA = |cross(dfx,dfy)|, where dfx = [dx;0;f(dx,0)-f(0,0)], dfy = [0;dy;f(0,dy)-f(0,0)]
  // taking limit dx->0, dy->0 gives A = int(sqrt(1+(df/dx)^2+(df/dy)^2))dxdy.
  Integrand.resize(N,N);
  for( int i=0; i<N; ++i )
	for( int j=0; j<N; ++j )
	  Integrand(i,j) = sqrt(1.+Derivative(0)(i,j)*Derivative(0)(i,j)+Derivative(1)(i,j)*Derivative(1)(i,j));

  // Integrate the function over the domain.
  return CCInt(Integrand,IntT);
}
