/* Construct the Weyl group for the given root system.
 * Input:
 * 	C : Cartan matrix from Cartan(name,d).
 * or
 * 	name : A, B, C, D, E, F, or G.
 * 	d : The number of dimensions.
 * Output:
 * 	The Weyl group for the given root system.
 *
 * Brett Ryland, April 2009
 */

#include "Chebyshev.hpp"
#include <boost/numeric/ublas/matrix_proxy.hpp>
#include <boost/numeric/ublas/vector_proxy.hpp>

// get all the matrices in the Weyl group (actually the transpose of the matrices),
IArray Weyl(IMatrix C)
{
  // W (d*d*n double) = Weyl group for the given Cartan matrix, n is the size of the group.
  int d,oldlength,newlength;
  IMatrix I;
  IArray gen, newgen, W;

  d=C.size1();
  I=boost::numeric::ublas::identity_matrix<int> (d);
  W=IArray(d+1); // 400 should be plenty for now... we will truncate it later
  W(0) = I;
  for( int n = 0; n<d; ++n) {
	W(n+1) = I;
	column(W(n+1),n) = column(W(n+1),n)-column(C,n);
  }
  gen = subrange(W,1,d+1);
  oldlength = 0; 
  newlength = 1+d;
  while( oldlength < newlength ) {
	// multiply all in list with generators
	for( int ndx=1; ndx<newlength; ++ndx ) {
	  newgen = gen;
	  int inclength=newgen.size();
	  for( int ndx2=0; ndx2<d; ++ndx2 ) {
		newgen(ndx2) = prod(W(ndx),newgen(ndx2));
	  }
	  // delete repeated entries
	  for( int k=d-1; k>=0; --k )
		for( unsigned int l = 0; l<W.size(); ++l ) 
		  if( norm_inf(W(l)-newgen(k)) == 0 ) {
			newgen.erase_element(k);
			--inclength;
			break;
		  }
	  W.resize(W.size()+inclength,true);
	  for( unsigned int i=0,j=W.size()-inclength; i<newgen.size(); ++i ) {
		if( newgen(i).size1()!=0 )
		  W(j++) = newgen(i);
	  }
	}
	oldlength = newlength;
	newlength = W.size();
	if( newlength>400 ) {
	  std::cerr << "Infinite group?" << std::endl;
	  return W;
	}
  }
  // check if -I is in W. In that case, sort the matrices
  for( unsigned int i=0; i<W.size(); ++i )
	if( norm_inf(W(i)+I) < 1e-7 ) {
	  i=1;
	  for( unsigned int j=1; j<W.size(); ++j ) {
		bool unique=true;
		for( unsigned int k=0; k<j; ++k ) {
		  if( norm_inf(W(j)+W(k)) < 1e-7 ) {
			unique=false;
			break;
		  }
		}
		if( unique==true )
		  W(i++)=W(j);
	  }
	  for( unsigned int i=0; i<W.size()/2; ++i )
		W(i+W.size()/2) = -W(i);
	  break;
	}
  return W;
}

IArray Weyl(char name, int d)
{
  return Weyl(Cartan(name,d));
}
