/* Construct the Cartan matrix for the specified root system.
 * Input:
 * 	name : A, B, C, D, E, F, or G.
 * 	d : The number of dimensions.
 * 	D (optional) : ?
 * Output:
 * 	The Cartan matrix for the specified root system.
 *
 * Brett Ryland, April 2009
 */

#include "Chebyshev.hpp"
#include <cmath>

IMatrix Cartan(char name, int d)
{
  DMatrix D(d,d);
  return Cartan(name,d,D);
}

IMatrix Cartan(char name, int d, DMatrix &D)
{
  IMatrix C;
  C.resize(d,d);
  D.resize(d,d);
  C.clear();
  D.clear();
  switch( name ){
	case 'A':
	  for( int j=0; j<d; ++j)
		C(j,j)=2;
	  for( int j=0; j<d-1; ++j) {
		C(j,j+1)=-1;
		C(j+1,j)=-1;
	  }
	  for( int j=0; j<d; ++j)
		D(j,j)=2.;
	  break;
	case 'B':
	  if( d < 2 ) {
		std::cerr << "Dimension must be > 1 ." << std::endl;
		return C;
	  }
	  for( int j=0; j<d; ++j)
		C(j,j)=2;
	  for( int j=0; j<d-2; ++j) {
		C(j,j+1)=-1;
		C(j+1,j)=-1;
	  }
	  C(d-1,d-2) = -1;
	  C(d-2,d-1) = -2;
	  for( int j=0; j<d-1; ++j)
		D(j,j)=2.;
	  D(d-1,d-1)=sqrt(2.);
	  break;
	case 'C':
	  if( d < 2 ) {
		std::cerr << "Dimension must be > 1 ." << std::endl;
		return C;
	  }
	  for( int j=0; j<d; ++j)
		C(j,j)=2;
	  for( int j=0; j<d-2; ++j) {
		C(j,j+1)=-1;
		C(j+1,j)=-1;
	  }
	  C(d-2,d-1)=-1;
	  C(d-1,d-2)=-2;
	  D(1,1)=sqrt(2.);
	  for( int j=0; j<d-1; ++j)
		D(j,j)=D(1,1);
	  D(d-1,d-1)=2.;
	  break;
	case 'D':
	  if( d < 4 ) {
		std::cerr << "Dimension must be > 3 ." << std::endl;
		return C;
	  }
	  for( int j=0; j<d; ++j)
		C(j,j)=2;
	  for( int j=0; j<d-2; ++j) {
		C(j,j+1)=-1;
		C(j+1,j)=-1;
	  }
	  C(d-2,d-1) = 0;
	  C(d-1,d-2) = 0;
	  C(d-3,d-1) = -1;
	  C(d-1,d-3) = -1;
	  for( int j=0; j<d; ++j)
		D(j,j)=2.;
	  break;
	case 'E':
	  if( (d < 5) || (d > 8) ) {
		std::cerr << "Dimension must be 6, 7 or 8." << std::endl;
		return C;
	  }
	  for( int j=0; j<d; ++j)
		C(j,j)=2;
	  for( int j=0; j<d-2; ++j) {
		C(j,j+1)=-1;
		C(j+1,j)=-1;
	  }
	  C(d-2,d-1) = 0;
	  C(d-1,d-2) = 0;
	  C(2,d-1) = -1;
	  C(d-1,2) = -1;
	  for( int j=0; j<d; ++j)
		D(j,j)=2.;
	  break;
	case 'F':
	  if( d != 4 ) {
		std::cerr << "Dimension must be 4." << std::endl;
		return C;
	  }
	  C(0,0)=2;C(0,1)=-1;
	  C(1,0)=-1;C(1,1)=2;C(1,2)=-2;
	  C(2,1)=-1;C(2,2)=2;C(2,3)=-1;
	  C(3,2)=-1;C(3,3)=2;
	  for( int j=0; j<d; ++j)
		D(j,j)=2.;
	  break;
	case 'G':
	  if( d != 2 ) {
		std::cerr << "Dimension must be 2." << std::endl;
		return C;
	  }
	  C(0,0)=2;C(0,1)=-1;C(1,0)=-3;C(1,1)=2;
	  D(0,0)=2./sqrt(3.);D(1,1)=2.;
	  break;
	default:
	  std::cerr << "Unknown Cartan matrix" << std::endl << "Possibilities are A 1.., B 2.., C 2.., D 4.., E 5..8, F 4, G 2" << std::endl;
	  return C;
  }
  return C;
}
