/* Compute the gradient of a function on the deltoid at the Chebyshev extremal points.
 * The gradient is computed w.r.t. the real (x_1,x_2) variables.
 * The general process is:
 *	- Find the coefficients a(k) by cosine FFT (by extending domain symmetrically).
 *	- Modify the a(k) so that the FFT is now the derivative via the recursion formula.
 *	- Apply an IFFT to recover the derivative at the internal points.
 * For polynomials of degree n, the gradient is exact if N>2n.
 *
 * Input:
 *	Values : Values of the function on the Chebyshev extremal points.
 *
 * Output:
 *	The gradient at the Chebyshev extremal points.
 *
 * Brett Ryland, April 2009
 */

#include "Chebyshev.hpp"
#include <complex>
#include <fftw3.h>
#include <boost/numeric/ublas/matrix_proxy.hpp>

#define imod(x,y) ((x)-(y)*int(floor(double(x)/(y))))

DArray Gradient(DMatrix Values)
{
  int N,gmax;
  IMatrix g;
  CMatrix A,a,bx,by;
  DArray Derivative;
  fftw_plan plan,plan_inv_x,plan_inv_y;

  N=Values.size1();
  if( Values.size2()!=(unsigned)N ) {
	std::cerr << "Input array is the wrong size" << std::endl;
	exit(1);
  }

  // get the dual fundamental domain and re-order it
  // note: the outermost entries of g could also be thrown away as b is zero for those g, but the computational gain is marginal
  g = dualfudo_sorted(N);
  gmax=norm_inf(row(g,0));

  a.resize(N,N/2+1);
  // perform a FFT on symmetrised Values to obtain the Chebyshev coefficients
  // plan the 2D FFT, maybe this could be improved by only planning it once?
  plan = fftw_plan_dft_r2c_2d(N, N, &(Values.data()[0]), reinterpret_cast<fftw_complex*>(&(a.data()[0])), FFTW_ESTIMATE);
  fftw_execute(plan);

  A.resize(gmax+2,gmax+2);
  A.clear();
  // pseudo Fourier coefficients, A is a inside the dual fundamental domain and zero outside
  for( unsigned int j=0; j<g.size2(); ++j ) {
	A(g(0,j),g(1,j))=a(g(0,j),g(1,j));
	A(g(1,j),g(0,j))=a(g(1,j),g(0,j));
  }

  // modify the Fourier coefficients via the recursion formula
  // |W|*k*A(k) = \sum_{l=1}^{2} \sum_{g\in W} b(k-g^{T}e_{l})_{l}*(g^{T}e_{l})
  if( (gmax+2>=N) || (norm_inf(row(g,1))+2>=N/2+1) ) { // expand bx and by slightly to allow recursion routine to work.
	bx.resize(N+2,N/2+3);
	by.resize(N+2,N/2+3);
  }
  else {
	bx.resize(N,N/2+1);
	by.resize(N,N/2+1);
  }
  bx.clear();
  by.clear();
  for( int j=(int)g.size2()-1; j>=0; --j ) {
	if( g(1,j)>0 )
	  bx(g(0,j),g(1,j))=3.*(g(0,j)+1)*A(g(0,j)+1,g(1,j))+bx(g(0,j)+2,g(1,j)-1)+by(g(0,j)+2,g(1,j))-by(g(0,j),g(1,j)+1);
	else // lower edge, b(m+1,-1)=b(m,1)
	  bx(g(0,j),0)=3.*(g(0,j)+1)*A(g(0,j)+1,0)+bx(g(0,j)+1,1)+by(g(0,j)+2,0)-by(g(0,j),1);
	if( g(0,j)>0 )
	  by(g(0,j),g(1,j))=3.*(g(1,j)+1)*A(g(0,j),g(1,j)+1)+by(g(0,j)-1,g(1,j)+2)+bx(g(0,j),g(1,j)+2)-bx(g(0,j)+1,g(1,j));
	else // origin
	  by(0,0)=3.*A(0,1)+by(1,1)+bx(0,2)-bx(1,0);
	// symmetry bx(m,n)=conj(by(n,m)) from a(m,n)=conj(a(n,m)) and symmetry of recursion formulae
	bx(g(1,j),g(0,j))=conj(by(g(0,j),g(1,j)));
	by(g(1,j),g(0,j))=conj(bx(g(0,j),g(1,j)));
  }
  if( (int)bx.size2()>N/2+1 ) {
	bx.resize(N,N/2+1,true);
	by.resize(N,N/2+1,true);
  }

  // apply symmetries (being careful of the reduced sizes of bx and by)
  for( int j=0; j<(int)g.size2(); ++j ) {
	if( imod(g(0,j)+g(1,j),N)<N/2+1 )
	  bx(imod(-g(0,j),N),imod(g(0,j)+g(1,j),N))=bx(g(0,j),g(1,j));
	if( imod(-g(1,j),N)<N/2+1 )
	  bx(imod(g(0,j)+g(1,j),N),imod(-g(1,j),N))=bx(g(0,j),g(1,j));
	bx(imod(-g(0,j)-g(1,j),N),g(0,j))=bx(g(0,j),g(1,j));
	if( imod(-g(0,j)-g(1,j),N)<N/2+1 )
	  bx(g(1,j),imod(-g(0,j)-g(1,j),N))=bx(g(0,j),g(1,j));
	if( imod(-g(0,j),N)<N/2+1 )
	  bx(imod(-g(1,j),N),imod(-g(0,j),N))=bx(g(0,j),g(1,j));
	if( imod(g(1,j)+g(0,j),N)<N/2+1 )
	  bx(imod(-g(1,j),N),imod(g(1,j)+g(0,j),N))=bx(g(1,j),g(0,j));
	if( imod(-g(0,j),N)<N/2+1 )
	  bx(imod(g(1,j)+g(0,j),N),imod(-g(0,j),N))=bx(g(1,j),g(0,j));
	bx(imod(-g(1,j)-g(0,j),N),g(1,j))=bx(g(1,j),g(0,j));
	if( imod(-g(1,j)-g(0,j),N)<N/2+1 )
	  bx(g(0,j),imod(-g(1,j)-g(0,j),N))=bx(g(1,j),g(0,j));
	if( imod(-g(1,j),N)<N/2+1 )
	  bx(imod(-g(0,j),N),imod(-g(1,j),N))=bx(g(1,j),g(0,j));
	if( imod(g(0,j)+g(1,j),N)<N/2+1 )
	  by(imod(-g(0,j),N),imod(g(0,j)+g(1,j),N))=by(g(0,j),g(1,j));
	if( imod(-g(1,j),N)<N/2+1 )
	  by(imod(g(0,j)+g(1,j),N),imod(-g(1,j),N))=by(g(0,j),g(1,j));
	by(imod(-g(0,j)-g(1,j),N),g(0,j))=by(g(0,j),g(1,j));
	if( imod(-g(0,j)-g(1,j),N)<N/2+1 )
	  by(g(1,j),imod(-g(0,j)-g(1,j),N))=by(g(0,j),g(1,j));
	if( imod(-g(0,j),N)<N/2+1 )
	  by(imod(-g(1,j),N),imod(-g(0,j),N))=by(g(0,j),g(1,j));
	if( imod(g(1,j)+g(0,j),N)<N/2+1 )
	  by(imod(-g(1,j),N),imod(g(1,j)+g(0,j),N))=by(g(1,j),g(0,j));
	if( imod(-g(0,j),N)<N/2+1 )
	  by(imod(g(1,j)+g(0,j),N),imod(-g(0,j),N))=by(g(1,j),g(0,j));
	by(imod(-g(1,j)-g(0,j),N),g(1,j))=by(g(1,j),g(0,j));
	if( imod(-g(1,j)-g(0,j),N)<N/2+1 )
	  by(g(0,j),imod(-g(1,j)-g(0,j),N))=by(g(1,j),g(0,j));
	if( imod(-g(1,j),N)<N/2+1 )
	  by(imod(-g(0,j),N),imod(-g(1,j),N))=by(g(1,j),g(0,j));
  }
  // multiply by J_x(z)^{-T}=[1 1;i -i] to get the Fourier coefficients of the Derivative in x coordinates
  for( int i=0; i<N; ++i )
	for( int j=0; j<N/2+1; ++j ) {
	  std::complex<double> I(0,1);
	  std::complex<double> temp[2];
	  temp[0]=bx(i,j);
	  temp[1]=by(i,j);
	  bx(i,j)+=temp[1];
	  by(i,j)=I*(temp[0]-temp[1]);
	}

  // take the 2D IFFT to recover the gradient
  Derivative.resize(2);
  Derivative(0).resize(N,N);
  Derivative(1).resize(N,N);
  // plan the 2D FFT, maybe this could be improved by only planning it once?
  plan_inv_x = fftw_plan_dft_c2r_2d(N, N, reinterpret_cast<fftw_complex*>(&(bx.data()[0])), &(Derivative(0).data()[0]), FFTW_ESTIMATE);
  plan_inv_y = fftw_plan_dft_c2r_2d(N, N, reinterpret_cast<fftw_complex*>(&(by.data()[0])), &(Derivative(1).data()[0]), FFTW_ESTIMATE);
  fftw_execute(plan_inv_x);
  fftw_execute(plan_inv_y);
  Derivative/=N*N; // normalise the FFT2

  // clean up
  fftw_destroy_plan(plan);
  fftw_destroy_plan(plan_inv_x);
  fftw_destroy_plan(plan_inv_y);

  return Derivative;
}
