/* Compute the gradient of a function on the standard triangle at the Chebyshev extremal points.
 * The gradient is computed w.r.t. the real (x_1,x_2) variables.
 * The general process is:
 *  - Find the coefficients a(k) by cosine FFT (by extending domain symmetrically).
 *  - Modify the a(k) so that the FFT is now the derivative via the recursion formula.
 *  - Apply an IFFT to recover the derivative at the internal points.
 * For polynomials of degree n, the gradient is exact if N>2n.
 *
 * Input:
 *  Values : Values of the function on the Chebyshev extremal points.
 * 	C : Triangle corners as a 2x3 matrix.
 *
 * Output:
 *  The gradient at the Chebyshev extremal points.
 *
 * Brett Ryland, April 2009
 */

#include "Chebyshev.hpp"

DArray TriGrad(DMatrix Values, DMatrix C)
{
  int N;
  double temp;
  Map map;
  map=ArbitraryTriangleToStandardTriangle(C);
  DMatrix J;
  DArray D;

  J.resize(2,2);
  J(0,0)=map.T(1,1);
  J(1,0)=-map.T(0,1);
  J(0,1)=-map.T(1,0);
  J(1,1)=map.T(0,0);
  J/=map.T(0,0)*map.T(1,1)-map.T(1,0)*map.T(0,1); // J=trans(inv(map.T));

  D=Gradient(Values);
  N=Values.size1();
  for( int i=0; i<N; ++i )
	for( int j=0; j<N; ++j ) {
	  temp=J(1,0)*D(0)(i,j)+J(1,1)*D(1)(i,j);
	  D(0)(i,j)=J(0,0)*D(0)(i,j)+J(0,1)*D(1)(i,j);
	  D(1)(i,j)=temp;
	}
  return D;
}
