/* Convert between some of the data types in Chebyshev.hpp and MATLAB data types.
 * In each case, the arguments are (in,ptr) where "in" is a datatype in Chebyshev.hpp
 * and "ptr" is a pointer to an element of "mxArray *plhs[]", e.g., array_conv(in,&plhs[0]);
 * Note that mxArray is column major, whereas c arrays are row major.
 *
 * Brett Ryland, April 2009
 */

#include "Chebyshev.hpp"
#include "mex.h"

#define max(x,y) ((x)>(y)?(x):(y))

void array_conv(DMatrix in, mxArray **ptr)
{
  double *out;
  *ptr=mxCreateDoubleMatrix(in.size1(),in.size2(),mxREAL);
  out=mxGetPr(*ptr);
  for( unsigned int i=0; i<in.size1(); ++i )
	for( unsigned int j=0; j<in.size2(); ++j )
	  out[i+j*in.size1()]=in(i,j);
}

void array_conv(DVector in, mxArray **ptr)
{
  double *out;
  *ptr=mxCreateDoubleMatrix(in.size(),1,mxREAL);
  out=mxGetPr(*ptr);
  for( unsigned int i=0; i<in.size(); ++i )
	out[i]=in(i);
}

void array_conv(IMatrix in, mxArray **ptr)
{
  double *out;
  *ptr=mxCreateDoubleMatrix(in.size1(),in.size2(),mxREAL);
  out=mxGetPr(*ptr);
  for( unsigned int i=0; i<in.size1(); ++i )
	for( unsigned int j=0; j<in.size2(); ++j )
	  out[i+j*in.size1()]=(double)in(i,j);
}

void array_conv(IVector in, mxArray **ptr)
{
  double *out;
  *ptr=mxCreateDoubleMatrix(in.size(),1,mxREAL);
  out=mxGetPr(*ptr);
  for( unsigned int i=0; i<in.size(); ++i )
	out[i]=(double)in(i);
}

void array_conv(CMatrix in, mxArray **ptr)
{
  double *outr,*outi;
  *ptr=mxCreateDoubleMatrix(in.size1(),in.size2(),mxCOMPLEX);
  outr=mxGetPr(*ptr);
  outi=mxGetPi(*ptr);
  for( unsigned int i=0; i<in.size1(); ++i )
	for( unsigned int j=0; j<in.size2(); ++j ) {
	  outr[i+j*in.size1()]=real(in(i,j));
	  outi[i+j*in.size1()]=imag(in(i,j));
	}
}

void array_conv(CVector in, mxArray **ptr)
{
  double *outr,*outi;
  *ptr=mxCreateDoubleMatrix(in.size(),1,mxREAL);
  outr=mxGetPr(*ptr);
  outi=mxGetPi(*ptr);
  for( unsigned int i=0; i<in.size(); ++i ) {
	outr[i]=real(in(i));
	outi[i]=imag(in(i));
  }
}

void array_conv(DArray in, mxArray **ptr)
{
  mwSize dims[3];
  double *out;
  dims[2]=in.size();
  dims[0]=in(0).size1();
  dims[1]=in(0).size2();
  *ptr=mxCreateNumericArray(3,dims,mxDOUBLE_CLASS,mxREAL);
  out=mxGetPr(*ptr);
  for( unsigned int i=0; i<dims[2]; ++i )
	for( unsigned int j=0; j<dims[0]; ++j )
	  for( unsigned int k=0; k<dims[1]; ++k )
		out[j+k*dims[0]+i*dims[0]*dims[1]]=in(i)(j,k);
}

void array_conv(IArray in, mxArray **ptr)
{
  mwSize dims[3];
  double *out;
  dims[2]=in.size();
  dims[0]=in(0).size1();
  dims[1]=in(0).size2();
  *ptr=mxCreateNumericArray(3,dims,mxDOUBLE_CLASS,mxREAL);
  out=mxGetPr(*ptr);
  for( unsigned int i=0; i<dims[2]; ++i )
	for( unsigned int j=0; j<dims[0]; ++j )
	  for( unsigned int k=0; k<dims[1]; ++k )
		out[j+k*dims[0]+i*dims[0]*dims[1]]=(double)in(i)(j,k);
}

void array_conv(JArray in, mxArray **ptr)
{
  mwSize dims[4];
  double *out;
  dims[2]=in.size1();
  dims[3]=in.size2();
  dims[0]=in(0,0).size1();
  dims[1]=in(0,0).size2();
  *ptr=mxCreateNumericArray(4,dims,mxDOUBLE_CLASS,mxREAL);
  out=mxGetPr(*ptr);
  for( unsigned int i=0; i<dims[2]; ++i )
	for( unsigned int j=0; j<dims[3]; ++j )
	  for( unsigned int k=0; k<dims[0]; ++k )
		for( unsigned int l=0; l<dims[1]; ++l )
		  out[k+l*dims[0]+i*dims[0]*dims[1]+j*dims[0]*dims[1]*dims[2]]=in(i,j)(k,l);
}

void array_conv(const mxArray **ptr, DMatrix &out)
{
  double *in;
  in=mxGetPr(*ptr);
  out.resize(mxGetM(*ptr),mxGetN(*ptr));
  for( unsigned int i=0; i<out.size1(); ++i )
	for( unsigned int j=0; j<out.size2(); ++j )
	  out(i,j)=in[i+j*out.size1()];
}

void array_conv(const mxArray **ptr, DVector &out)
{
  double *in;
  in=mxGetPr(*ptr);
  out.resize(max(mxGetM(*ptr),mxGetN(*ptr)));
  for( unsigned int i=0; i<out.size(); ++i )
	out(i)=in[i];
}

void array_conv(const mxArray **ptr, IMatrix &out)
{
  double *in;
  in=mxGetPr(*ptr);
  out.resize(mxGetM(*ptr),mxGetN(*ptr));
  for( unsigned int i=0; i<out.size1(); ++i )
	for( unsigned int j=0; j<out.size2(); ++j )
	  out(i,j)=int(in[i+j*out.size1()]);
}

void array_conv(const mxArray **ptr, IVector &out)
{
  double *in;
  in=mxGetPr(*ptr);
  out.resize(max(mxGetM(*ptr),mxGetN(*ptr)));
  for( unsigned int i=0; i<out.size(); ++i )
	out(i)=int(in[i]);
}

void array_conv(const mxArray **ptr, CMatrix &out)
{
  double *inr,*ini;
  inr=mxGetPr(*ptr);
  ini=mxGetPi(*ptr);
  out.resize(mxGetM(*ptr),mxGetN(*ptr));
  for( unsigned int i=0; i<out.size1(); ++i )
	for( unsigned int j=0; j<out.size2(); ++j ) {
	  real(out(i,j))=inr[i+j*out.size1()];
	  imag(out(i,j))=ini[i+j*out.size1()];
	}
}

void array_conv(const mxArray **ptr, CVector &out)
{
  double *inr,*ini;
  inr=mxGetPr(*ptr);
  ini=mxGetPi(*ptr);
  out.resize(max(mxGetM(*ptr),mxGetN(*ptr)));
  for( unsigned int i=0; i<out.size(); ++i ) {
	real(out(i))=inr[i];
	imag(out(i))=ini[i];
  }
}

void array_conv(const mxArray **ptr, DArray &out)
{
  const mwSize *dims;
  double *in;
  in=mxGetPr(*ptr);
  dims=mxGetDimensions(*ptr);
  out.resize(dims[2]);
  for( unsigned int i=0; i<dims[2]; ++i ) {
	out(i).resize(dims[0],dims[1]);
	for( unsigned int j=0; j<dims[0]; ++j )
	  for( unsigned int k=0; k<dims[1]; ++k )
		out(i)(j,k)=in[j+k*dims[0]+i*dims[0]*dims[1]];
  }
}

void array_conv(const mxArray **ptr, IArray &out)
{
  const mwSize *dims;
  double *in;
  in=mxGetPr(*ptr);
  dims=mxGetDimensions(*ptr);
  out.resize(dims[2]);
  for( unsigned int i=0; i<dims[2]; ++i ) {
	out(i).resize(dims[0],dims[1]);
	for( unsigned int j=0; j<dims[0]; ++j )
	  for( unsigned int k=0; k<dims[1]; ++k )
		out(i)(j,k)=int(in[j+k*dims[0]+i*dims[0]*dims[1]]);
  }
}
