/* Information on orbits of a matrix group acting on a periodic 2D lattice.
 * Input:
 * 	G : The matrix group acting on the 2D lattice (the first element of G must be the identity).
 * 	fudo : The dual fundamental domain.
 * 	N : Periodicity of the lattice (assumed equal in both dimensions).
 * 	orbitnumber (optional) : For each lattice point k = [k1;k2],
 * 		orbitnumber(k1,k2) returns the column index of the corresponding point in fudo.
 * 	orbitaction (optional) : For each lattice point k = [k1;k2], 
 *      orbitaction(k1,k2) returns the number of _an_ element of G, such that
 *      this element applied to the fudo point of the orbit yields k. (Defined up to isotropy).
 *      orbitaction(k1,k2) == 1 iff [k1;k2] \in fudo
 * 	isotropy (optional) : For each point j in fudo, column j of isotropy gives the isotropy subgroup
 * 		of G at this point. The number -1 is used as fill_in to get a full matrix.
 * Output:
 * 	The size of the isotropy subgroups.
 *
 * Brett Ryland, April 2009
 */

#include "Chebyshev.hpp"
#include <boost/numeric/ublas/matrix_proxy.hpp>

IVector orbit_info(IArray G, IMatrix fudo, int N)
{
  IMatrix orbnum, orbact, iso;
  return orbit_info(G,fudo,N,orbnum,orbact,iso);
}

IVector orbit_info(IArray G, IMatrix fudo, int N, IMatrix &orbitnumber, IMatrix &orbitaction, IMatrix &isotropy)
{
  using namespace boost::numeric::ublas;
  int grpsize=G.size();
  int nfudo=fudo.size2();
  IVector fixpts,Niso;
  IMatrix gk,temp;

  orbitnumber.resize(N,N);
  orbitaction.resize(N,N);
  isotropy=scalar_matrix<int> (grpsize,nfudo,-1); // initialise it to -1
  Niso.resize(nfudo);
  Niso.clear();
  fixpts.resize(nfudo);
  fixpts.clear();
  gk.resize(fudo.size1(),fudo.size2());
  temp.resize(G(0).size1(),G(0).size2());
  for( int igr=0; igr<grpsize; ++igr) {
	int nfixpts=0;
	temp=G(igr);
	gk=prod(temp,fudo); // action of group element on fudo points
	for( unsigned int i=0; i<gk.size1(); ++i)
	  for( unsigned int j=0; j<gk.size2(); ++j)
		gk(i,j)=gk(i,j)-N*(int)floor((double)gk(i,j)/N);
	for( unsigned int i=0; i<fudo.size2(); ++i)
	  if( norm_1(column(gk,i)-column(fudo,i))==0 )
		fixpts(nfixpts++)=i;
	for( int i=0; i<nfixpts; ++i)
	  ++Niso(fixpts(i));
	for( int ndx=0; ndx<nfixpts; ++ndx )
	  isotropy(Niso(fixpts(ndx))-1,fixpts(ndx)) = igr;
	for( int ndx=0; ndx<nfudo; ++ndx) {
	  orbitnumber(gk(0,ndx),gk(1,ndx)) = ndx;
	  orbitaction(gk(0,ndx),gk(1,ndx)) = igr;
	}
  }
  // for points in fudo, we want orbit action to return 0 (Identity)
  for( int ndx=0; ndx<nfudo; ++ndx)
	orbitaction(fudo(0,ndx),fudo(1,ndx)) = 0;
  return Niso;
}
