/* Find the indices of the points that are interior to the triangle C.
 * The points are tested by projecting them onto the normals of the edge vectors
 * of the triangle and checking the sign of the magnitudes.
 * See http://www.scribd.com/doc/2941109/Constructing-A-PointInTriangle-Matrix
 *
 * Input:
 *	points : A list of points to test.
 * 	C : Triangle corners as a 2x3 matrix.
 *	flags (optional) : [interior edge corner] indicates which set of points to look for.
 *		Set a flag to 1 to include those points or 0 to exclude them. (Default is [1 1 1].)
 *	tol (optional) : Tolerance of edge detection. (Default is 1e-13.)
 *
 * Output:
 *	The indices of the points that are inside the triangle.
 *
 * Brett Ryland, April 2009
 */

#include "Chebyshev.hpp"
#include <boost/numeric/ublas/matrix_proxy.hpp>

IVector triangle_interior_points(DMatrix points, DMatrix C)
{
  double tol=1e-13; // default tolerance for edge detection
  IVector flags(4);
  flags(0)=1;
  flags(1)=1;
  flags(2)=1;
  flags(3)=0;
  return triangle_interior_points(points, C, flags, tol);
}

IVector triangle_interior_points(DMatrix points, DMatrix C, IVector flags)
{
  double tol=1e-13; // default tolerance for edge detection
  return triangle_interior_points(points, C, flags, tol);
}

IVector triangle_interior_points(DMatrix points, DMatrix C, IVector flags, double tol)
{
  /* general 3D implementation
   * A=[C.' zeros(3,1)];
   * B=[-1 1 0;0 -1 1;1 0 -1]*A;
   * n=cross(B(1,:),B(2,:));
   * D=B*[0 -n(3) n(1);n(3) 0 -n(1);-n(2) n(1) 0];
   * G=[D -dot(A,D,2)];
   * ndx=find(abs(sum(sign(G*[points;zeros(1,size(points,2));ones(1,size(points,2))])))==3);
   */
  using namespace boost::numeric::ublas;
  double n;
  IVector ndx;
  IMatrix sgn;
  DMatrix B(3,2),G(3,3);

  if( points.size1()!=2 )
	points=trans(points);
  if( points.size1()!=2 ) {
	std::cerr << "Input variable \"points\" must be a 2xn or nx2 array." << std::endl;
	return ndx;
  }
  points.resize(3,points.size2(),true);
  row(points,2)=scalar_vector<double> (points.size2(),1.);
  // optimised 2D implementation
  C=trans(C);
  row(B,0)=row(C,1)-row(C,0);
  row(B,1)=row(C,2)-row(C,1);
  row(B,2)=row(C,0)-row(C,2);
  n=B(0,0)*B(1,1)-B(0,1)*B(1,0);
  column(G,0)=(double)n*column(B,1);
  column(G,1)=-(double)n*column(B,0);
  column(G,2)=-element_prod(column(C,0),column(G,0))-element_prod(column(C,1),column(G,1));
  points=prod(G,points);
  sgn.resize(3,points.size2());
  for( unsigned int i=0; i<sgn.size1(); ++i)
	for( unsigned int j=0; j<sgn.size2(); ++j)
	  if( fabs(points(i,j))<tol )
		sgn(i,j)=0; // tolerance to being on an edge
	  else
		if( points(i,j)>0. )
		  sgn(i,j)=1;
		else
		  sgn(i,j)=-1;
  ndx.resize(points.size2());
  for( unsigned int i=0, j=0; i<points.size2(); ++i) {
	if( (flags(0)==1) && (abs(sum(column(sgn,i)))==3) ) // ==3 for interior
	  ndx(j++)=i;
	if( (flags.size()>1) && (flags(1)==1) && (abs(sum(column(sgn,i)))==2) ) // ==2 for edges
	  ndx(j++)=i;
	if( (flags.size()>2) && (flags(2)==1) && (norm_1(column(sgn,i))==1) ) // corners
	  ndx(j++)=i;
	if( (flags.size()>3) && (flags(3)==1) && (abs(sum(column(sgn,i)))==0) ) // ==0 for edges outside the triangle
	  ndx(j++)=i;
	if( i==points.size2()-1 )
	  ndx.resize(j,true);
  }
  return ndx;
}
