/* Construct the overlap information from the triangles and downscaling.
 * Adjust mod(,1) statements for desired geometry.
 *
 * Input:
 *	C : Corners of the triangles.
 *	N : Downscaling factor.
 *
 * Output:
 *	The overlap information.
 *		overlap.inner(j1,j2).Points = coordinates of points from deltoid j2 in triangle j1 (when j2=j1, this only contains the points that are in the triangle due to wrapping of the domain)
 *		overlap.inner(j1,j2).Index = index of corresponding points in overlap(j1,j2).Points
 *		overlap.edges(e) = edge overlap information: [triangle indices] of points on edges of triangles
 *			should be nx2, where n is 1 for an external edge or 2 for an internal one
 *		overlap.corners(c) = corner overlap information: [triangle indices] of points on corners of triangles
 *			should be nx2 in general, where n is the number of coinciding corners,
 *			but may be nx4 if a triangle wraps around the domain, e.g., C(0)=[0 0 1;0 1 0]
 *
 * Brett Ryland, April 2009
 */

#include "Chebyshev_Spectral.hpp"
#include <cfloat>
#include <boost/numeric/ublas/matrix_proxy.hpp>

// tolerance for detecting coinciding edge and corner points
#define tol 1e-10
// wrapping function. Define it as you like.
// #define wrap(z) ((z+tol)-floor(z+tol))-tol
#define wrap(z) (z)

Overlap Construct_Overlap(DArray C, int N)
{
  using namespace boost::numeric::ublas;
  int d;
  IVector local_points,edge_points,corner_points,flags(4);
  DVector theta,ones;
  DMatrix z,Z,cnrs,edpts;
  Overlap overlap;
  Map map;

  ones=scalar_vector<double> (N*N,1.);
  d=C.size();
  theta.resize(N);
  for( int i=0; i<N; ++i)
	theta(i) = i*2.*M_PI/N;
  Z.resize(2,N*N);
  for( int i=0; i<N; ++i)
	for( int j=0; j<N; ++j) {
	  Z(0,i*N+j) = (cos(theta(i)) + cos(theta(j)) + cos(theta(i)-theta(j)))/3.;
	  Z(1,i*N+j) = (sin(theta(i)) - sin(theta(j)) - sin(theta(i)-theta(j)))/3.;
	  if( fabs(Z(0,i*N+j)) < DBL_EPSILON )
		Z(0,i*N+j)=0.;
	  if( fabs(Z(1,i*N+j)) < DBL_EPSILON )
		Z(1,i*N+j)=0.;
	}
  // indices to points inside your own triangle are the same for all triangles
  map=ArbitraryTriangleToStandardTriangle(C(0));
  z.resize(2,N*N);
  z=prod(map.T,Z)+outer_prod(map.c,ones);
  local_points=triangle_interior_points(z,C(0));
  flags.clear(); flags(2)=1;
  corner_points=triangle_interior_points(z,C(0),flags);
  flags.clear(); flags(1)=1; flags(3)=0;
  edge_points=triangle_interior_points(z,C(0),flags);

  // find coinciding corner points
  if( corner_points.size()>0 ) {
	// get a unique list of the corners of all triangles
	cnrs.resize(3*d,2);
	for( int i=0; i<d; ++i ) {
	  cnrs(3*i,0)=wrap(C(i)(0,0));
	  cnrs(3*i,1)=wrap(C(i)(1,0));
	  cnrs(3*i+1,0)=wrap(C(i)(0,1));
	  cnrs(3*i+1,1)=wrap(C(i)(1,1));
	  cnrs(3*i+2,0)=wrap(C(i)(0,2));
	  cnrs(3*i+2,1)=wrap(C(i)(1,2));
	}
	uniq(cnrs,tol);
	overlap.corners.resize(cnrs.size1());
	for( unsigned int j1=0; j1<overlap.corners.size(); ++j1 ) {
	  int numcnrs=0,numpts;
	  overlap.corners(j1).resize(d);
	  for( int j2=0; j2<d; ++j2 ) { // contributions to corner j1 from triangle j2
		IVector ci;
		map=ArbitraryTriangleToStandardTriangle(C(j2));
		z=prod(map.T,Z)+outer_prod(map.c,ones);
		for( unsigned int k=0; k<corner_points.size(); ++k )
		  column(z,k)=column(z,corner_points(k));
		z.resize(2,corner_points.size(),true);
		z=trans(z);
		ci=uniq(z,tol); // sorted unique corner points with tolerance tol
		numpts=0;
		for( unsigned int k=0; k<z.size1(); ++k )
		  if( fabs(wrap(z(k,0)-cnrs(j1,0)))+fabs(wrap(z(k,1)-cnrs(j1,1)))<tol ) {
			if( numpts==0 ) {
			  overlap.corners(j1)(numcnrs).resize(4); // triangles only have 3 corners
			  overlap.corners(j1)(numcnrs)(numpts++)=j2;
			}
			overlap.corners(j1)(numcnrs)(numpts++)=corner_points(ci(k));
		  }
		if( numpts>0 )
		  overlap.corners(j1)(numcnrs++).resize(numpts,true);
	  }
	  overlap.corners(j1).resize(numcnrs,true);
	}
  }
  else {
	overlap.corners.resize(0);
  }

  // find coinciding edge points
  if( edge_points.size()>0 ) {
	edpts.resize(d*edge_points.size(),2);
	for( int j=0; j<d; ++j ) {
	  map=ArbitraryTriangleToStandardTriangle(C(j));
	  z=prod(map.T,Z)+outer_prod(map.c,ones);
	  for( unsigned int k=0; k<edge_points.size(); ++k )
		column(z,k)=column(z,edge_points(k));
	  z.resize(2,edge_points.size(),true);
	  for( unsigned int k=0; k<edge_points.size(); ++k ) {
		edpts(j*edge_points.size()+k,0)=wrap(z(0,k));
		edpts(j*edge_points.size()+k,1)=wrap(z(1,k));
	  }
	}
	uniq(edpts,tol); // unique list of the edge points of all triangles
	overlap.edges.resize(edpts.size1());
	for( unsigned int j1=0; j1<overlap.edges.size(); ++j1 ) {
	  int triangle=0;
	  overlap.edges(j1).resize(2); // only two triangles can share a point along an edge
	  for( int j2=0; j2<d; ++j2 ) {
		IVector ei;
		map=ArbitraryTriangleToStandardTriangle(C(j2));
		z=prod(map.T,Z)+outer_prod(map.c,ones);
		for( unsigned int k=0; k<edge_points.size(); ++k )
		  column(z,k)=column(z,edge_points(k));
		z.resize(2,edge_points.size(),true);
		z=trans(z);
		ei=uniq(z,tol); // sorted unique edge points of triangle j2 with tolerance tol
		for( unsigned int k=0; k<z.size1(); ++k ) {
		  if( fabs(wrap(z(k,0))-edpts(j1,0))+fabs(wrap(z(k,1))-edpts(j1,1))<tol ) {
			overlap.edges(j1)(triangle).resize(2); // each triangle can only have one point here
			overlap.edges(j1)(triangle)(0)=j2;
			overlap.edges(j1)(triangle++)(1)=edge_points(ei(k));
			break;
		  }
		}
		if( triangle==2 )
		  break; // only two triangles can share a point along an edge
	  }
	  if( triangle==1 )
		overlap.edges(j1).resize(1,true); // external edge point (only exists in one triangle)
	}
  }
  else {
	overlap.edges.resize(0);
  }

  overlap.inner.resize(d,d);
  // return overlap; // skip the overlapping points
  // find points in the triangle from other deltoids
  for( int j1=0; j1<d; ++j1 )
	for( int j2=0; j2<d; ++j2 ) {
	  map=ArbitraryTriangleToStandardTriangle(C(j2));
	  z=prod(map.T,Z)+outer_prod(map.c,ones);
	  for( unsigned int i=0; i<z.size2(); ++i ) {
		z(0,i)=wrap(z(0,i));
		z(1,i)=wrap(z(1,i));
	  }
	  overlap.inner(j1,j2).Index=setdiff(triangle_interior_points(z,C(j1)),local_points); // index of points in deltoid
	  overlap.inner(j1,j2).Points.resize(2,overlap.inner(j1,j2).Index.size());
	  for( unsigned int i=0; i<overlap.inner(j1,j2).Index.size(); ++i) {
		overlap.inner(j1,j2).Points(0,i)=wrap(z(0,overlap.inner(j1,j2).Index(i))); // x coord in triangle
		overlap.inner(j1,j2).Points(1,i)=wrap(z(1,overlap.inner(j1,j2).Index(i))); // y coord in triangle
	  }
	}

  return overlap;
}
