/* On a triangular decomposition of a domain the deltoids overlap, thus some points in each deltoid are
 * some other triangle(s). These points can be made to agree with the data from those triangles by
 * interpolating these points on the triangles.
 *
 * Input:
 *	overlap : index of which points belong to which deltoid and are in which triangle
 *	C : triangle corners
 *	in : values of the function to be smoothed
 *	maxIter : maximum number of iterations (0 for unlimited)
 *
 * Output:
 *	out (NxNxn) = smooth values of the function
 *
 * Brett Ryland, April 2009
 */

#include "Chebyshev_Spectral.hpp"
#include <omp.h>
#include <boost/numeric/ublas/matrix_proxy.hpp>
#include <boost/numeric/ublas/vector_proxy.hpp>

#define imod(x,y) ((x)-(y)*(int)floor((double)(x)/(y)))
#define max(x,y) (x)>(y)?(x):(y)

DArray Smooth_Overlap(Overlap overlap, DArray C, DArray in)
{
  return Smooth_Overlap(overlap,C,in,0);
}

DArray Smooth_Overlap(Overlap overlap, DArray C, DArray in, int maxIter)
{
  using namespace boost::numeric::ublas;
  int n,N,iter;
  double err,err_last;
  IArray W;
  DArray out(in);

  n=in.size();
  N=in(0).size1();
  W=Weyl('A',2);

  // average the corners
#pragma omp parallel for
  for( int j=0; j<(int)overlap.corners.size(); ++j ) {
	int count=0;
	double avg=0.;
	for( unsigned int j2=0; j2<overlap.corners(j).size(); ++j2 )
	  for( unsigned int j3=1; j3<overlap.corners(j)(j2).size(); ++j3 ) {
		avg+=in(overlap.corners(j)(j2)(0))(overlap.corners(j)(j2)(j3)/N,overlap.corners(j)(j2)(j3)%N);
		++count;
	  }
	avg/=double(count);
	for( unsigned int j2=0; j2<overlap.corners(j).size(); ++j2 )
	  for( unsigned int j3=1; j3<overlap.corners(j)(j2).size(); ++j3 ) {
		IVector k(2);
		k(0)=overlap.corners(j)(j2)(j3)/N;
		k(1)=overlap.corners(j)(j2)(j3)%N;
		for( unsigned int j4=0; j4<W.size(); ++j4 )
		  out(overlap.corners(j)(j2)(0))(imod(W(j4)(0,0)*k(0)+W(j4)(1,0)*k(1),N),imod(W(j4)(0,1)*k(0)+W(j4)(1,1)*k(1),N))=avg;
	  }
  }

  // average the edges
#pragma omp parallel for
  for( int j=0; j<(int)overlap.edges.size(); ++j ) {
	int count=0;
	double avg=0.;
	for( unsigned int j2=0; j2<overlap.edges(j).size(); ++j2 ) {
	  avg+=in(overlap.edges(j)(j2)(0))(overlap.edges(j)(j2)(1)/N,overlap.edges(j)(j2)(1)%N);
	  ++count;
	}
	avg/=double(count);
	for( unsigned int j2=0; j2<overlap.edges(j).size(); ++j2 ) {
	  IVector k(2);
	  k(0)=overlap.edges(j)(j2)(1)/N;
	  k(1)=overlap.edges(j)(j2)(1)%N;
	  for( unsigned int j3=0; j3<W.size(); ++j3 )
		out(overlap.edges(j)(j2)(0))(imod(W(j3)(0,0)*k(0)+W(j3)(1,0)*k(1),N),imod(W(j3)(0,1)*k(0)+W(j3)(1,1)*k(1),N))=avg;
	}
  }

  // return out; // don't do the interpolation, just the edge averaging
  // for each triangle (j1) and deltoid (j2), interpolate all the values that lie inside the triangle from the deltoid
  iter=0;err=0.;err_last=0.;
  while( (iter<2) || (err<err_last) ) {
	double norminf;
	++iter;
	in=out;
// this seems to break fftw_plan_dft_r2c_2d in Interp.cpp
// #pragma omp parallel for
	for( int j1=0; j1<n; ++j1 ) {
	  int numpoints,ndx;
	  IVector oli; // overlap index
	  DVector olv; // overlap values
	  DMatrix olp; // overlap points
	  numpoints=0;
	  for( int j2=0; j2<n; ++j2 )
		numpoints+=overlap.inner(j1,j2).Index.size();
	  oli.resize(numpoints);
	  olp.resize(2,numpoints);
	  ndx=0;
	  for( int j2=0; j2<n; ++j2 )
		if( overlap.inner(j1,j2).Index.size() > 0 ) {
		  subslice(oli,ndx,1,overlap.inner(j1,j2).Index.size())=overlap.inner(j1,j2).Index;
		  subslice(olp,0,1,2,ndx,1,overlap.inner(j1,j2).Index.size())=overlap.inner(j1,j2).Points;
		  ndx+=overlap.inner(j1,j2).Index.size();
		}
	  ndx=0;
	  olv=Interp(olp,in(j1),C(j1));
	  for( int j2=0; j2<n; ++j2 ) {
		for( unsigned int i=0; i<overlap.inner(j1,j2).Index.size(); ++i )
		  out(j2).data()[oli(ndx+i)]=olv(ndx+i);
		ndx+=overlap.inner(j1,j2).Index.size();
	  }
	}
	err_last=err; err=0.; norminf=0.;
	for( int j1=0; j1<n; ++j1 )
	  for( int j2=0; j2<n; ++j2 )
		for( unsigned int j3=0; j3<overlap.inner(j1,j2).Index.size(); ++j3 ) {
		  err=max(err,fabs(out(j2).data()[overlap.inner(j1,j2).Index(j3)]-in(j2).data()[overlap.inner(j1,j2).Index(j3)]));
		  norminf=max(norminf,fabs(out(j2).data()[overlap.inner(j1,j2).Index(j3)]));
		}
	if( norminf<1e-20 )
	  break; // in case the solution is 0 and this loop continues converging forever (very unlikely)
	if( (maxIter!=0) && (iter>=maxIter) )
	  break;
  }

  return out;
}
