/* Convert between some of the data types in Chebyshev.hpp and MATLAB data types.
 * In each case, the arguments are (in,ptr) where "in" is a datatype in Chebyshev.hpp
 * and "ptr" is a pointer to an element of "mxArray *plhs[]", e.g., array_conv(in,&plhs[0]);
 * Note that mxArray is column major, whereas c arrays are row major.
 *
 * Brett Ryland, April 2009
 */

#include <mex.h>
#include "Chebyshev_Spectral.hpp"
#include "array_conv_Spectral.hpp"

#define max(x,y) ((x)>(y)?(x):(y))

void array_conv(Overlap in, mxArray **ptr, int N)
{
  mwSize dims[3];
  double *pts,*ndx;
  mxArray *points,*index;
  dims[0]=in.inner.size1();
  dims[1]=in.inner.size2();
  dims[2]=2;
  *ptr=mxCreateCellArray(3,dims);
  for( unsigned int i=0; i<dims[0]; ++i )
	for( unsigned int j=0; j<dims[1]; ++j ) {
	  points=mxCreateDoubleMatrix(2,in.inner(i,j).Index.size(),mxREAL);
	  index=mxCreateDoubleMatrix(1,in.inner(i,j).Index.size(),mxREAL);
	  pts=mxGetPr(points);
	  ndx=mxGetPr(index);
	  for( unsigned int k=0; k<in.inner(i,j).Index.size(); ++k ) {
		pts[2*k]=in.inner(i,j).Points(0,k);
		pts[2*k+1]=in.inner(i,j).Points(1,k);
		ndx[k]=in.inner(i,j).Index(k)+1;
	  }
	  mxSetCell(*ptr,i+j*dims[0],points);
	  mxSetCell(*ptr,i+j*dims[0]+dims[0]*dims[1],index);
	}
}

void array_conv(Overlap in, mxArray **inner, mxArray **edges, mxArray **corners, int N)
{
  mwSize dims[3];
  double *pts,*ndx;
  mxArray *points,*index;
  dims[0]=in.inner.size1();
  dims[1]=in.inner.size2();
  dims[2]=2;
  *inner=mxCreateCellArray(3,dims);
  for( unsigned int i=0; i<dims[0]; ++i )
	for( unsigned int j=0; j<dims[1]; ++j ) {
	  points=mxCreateDoubleMatrix(2,in.inner(i,j).Index.size(),mxREAL);
	  index=mxCreateDoubleMatrix(1,in.inner(i,j).Index.size(),mxREAL);
	  pts=mxGetPr(points);
	  ndx=mxGetPr(index);
	  for( unsigned int k=0; k<in.inner(i,j).Index.size(); ++k ) {
		int tmp;
		pts[2*k]=in.inner(i,j).Points(0,k);
		pts[2*k+1]=in.inner(i,j).Points(1,k);
		tmp=in.inner(i,j).Index(k);
		ndx[k]=(tmp%N)*N+tmp/N+1;
	  }
	  mxSetCell(*inner,i+j*dims[0],points);
	  mxSetCell(*inner,i+j*dims[0]+dims[0]*dims[1],index);
	}
  *edges=mxCreateCellMatrix(in.edges.size(),1);
  for( unsigned int i=0; i<in.edges.size(); ++i ) {
	dims[0]=in.edges(i).size();
	dims[1]=0;
	for( unsigned int j=0; j<dims[0]; ++j )
	  dims[1]=max(dims[1],in.edges(i)(j).size());
	index=mxCreateDoubleMatrix(dims[0],dims[1],mxREAL);
	ndx=mxGetPr(index);
	for( unsigned int j=0; j<in.edges(i).size(); ++j ) {
	  if( in.edges(i)(j).size()!=0 )
		ndx[j]=in.edges(i)(j)(0)+1;
	  for( unsigned int k=1; k<in.edges(i)(j).size(); ++k ) {
		int tmp=in.edges(i)(j)(k);
		ndx[j+k*dims[0]]=(tmp%N)*N+tmp/N+1;
	  }
	}
	mxSetCell(*edges,i,index);
  }
  *corners=mxCreateCellMatrix(in.corners.size(),1);
  for( unsigned int i=0; i<in.corners.size(); ++i ) {
	dims[0]=in.corners(i).size();
	dims[1]=0;
	for( unsigned int j=0; j<dims[0]; ++j )
	  dims[1]=max(dims[1],in.corners(i)(j).size());
	index=mxCreateDoubleMatrix(dims[0],dims[1],mxREAL);
	ndx=mxGetPr(index);
	for( unsigned int j=0; j<dims[0]; ++j ) {
	  if( in.corners(i)(j).size()!=0 )
		ndx[j]=in.corners(i)(j)(0)+1;
	  for( unsigned int k=1; k<in.corners(i)(j).size(); ++k ) {
		int tmp=in.corners(i)(j)(k);
		ndx[j+k*dims[0]]=(tmp%N)*N+tmp/N+1;
	  }
	}
	mxSetCell(*corners,i,index);
  }
}

void array_conv(const mxArray **ptr, Overlap &out, int N)
{
  const mwSize *dims;
  double *pts,*ndx;
  mxArray *points,*index;
  dims=mxGetDimensions(*ptr);
  out.inner.resize(dims[0],dims[1]);
  for( unsigned int i=0; i<dims[0]; ++i )
	for( unsigned int j=0; j<dims[1]; ++j ) {
	  points=mxGetCell(*ptr,i+j*dims[0]);
	  index=mxGetCell(*ptr,i+j*dims[0]+dims[0]*dims[1]);
	  pts=mxGetPr(points);
	  ndx=mxGetPr(index);
	  out.inner(i,j).Points.resize(2,mxGetN(index));
	  out.inner(i,j).Index.resize(mxGetN(index));
	  for( unsigned int k=0; k<out.inner(i,j).Index.size(); ++k ) {
		out.inner(i,j).Points(0,k)=pts[2*k];
		out.inner(i,j).Points(1,k)=pts[2*k+1];
		out.inner(i,j).Index(k)=int(ndx[k])-1;
	  }
	}
}

void array_conv(const mxArray **inner, const mxArray **edges, const mxArray **corners, Overlap &out, int N)
{
  const mwSize *dims;
  mwSize dim[2];
  double *pts,*ndx;
  mxArray *points,*index;
  dims=mxGetDimensions(*inner);
  out.inner.resize(dims[0],dims[1]);
  for( unsigned int i=0; i<dims[0]; ++i )
	for( unsigned int j=0; j<dims[1]; ++j ) {
	  points=mxGetCell(*inner,i+j*dims[0]);
	  index=mxGetCell(*inner,i+j*dims[0]+dims[0]*dims[1]);
	  pts=mxGetPr(points);
	  ndx=mxGetPr(index);
	  out.inner(i,j).Points.resize(2,mxGetN(index));
	  out.inner(i,j).Index.resize(mxGetN(index));
	  for( unsigned int k=0; k<out.inner(i,j).Index.size(); ++k ) {
		out.inner(i,j).Points(0,k)=pts[2*k];
		out.inner(i,j).Points(1,k)=pts[2*k+1];
		out.inner(i,j).Index(k)=int(ndx[k])-1;
	  }
	}
  out.edges.resize(mxGetM(*edges));
  for( unsigned int i=0; i<out.edges.size(); ++i ) {
	index=mxGetCell(*edges,i);
	ndx=mxGetPr(index);
	dim[0]=mxGetM(index);
	dim[1]=mxGetN(index);
	out.edges(i).resize(dim[0]);
	for( unsigned int j=0; j<dim[0]; ++j ) {
	  out.edges(i)(j).resize(dim[1]);
	  for( unsigned int k=0; k<dim[1]; ++k )
		if( int(ndx[k+j*dim[1]])!=0 )
		  out.edges(i)(j)(k)=int(ndx[k+j*dim[1]])-1;
		else {
		  out.edges(i)(j).resize(k+1,true);
		  break;
		}
	}
  }
  out.corners.resize(mxGetM(*corners));
  for( unsigned int i=0; i<out.corners.size(); ++i ) {
	index=mxGetCell(*corners,i);
	ndx=mxGetPr(index);
	dim[0]=mxGetM(index);
	dim[1]=mxGetN(index);
	out.corners(i).resize(dim[0]);
	for( unsigned int j=0; j<dim[0]; ++j ) {
	  out.corners(i)(j).resize(dim[1]);
	  for( unsigned int k=0; k<dim[1]; ++k )
		if( int(ndx[k+j*dim[1]])!=0 )
		  out.corners(i)(j)(k)=int(ndx[k+j*dim[1]])-1;
		else {
		  out.corners(i)(j).resize(k+1,true);
		  break;
		}
	}
  }
}
