/* Create a spectral element method on the 2-torus using multivariate Chebyshev polynomials
 * Heat equation with forward Euler time-stepping.
 * To make a movie, uncomment the lines containing "mov"
 */
#include <iostream>
#include <cfloat>
#include <csignal> // signal stuff only works in linux
#include <mex.h> // mex.h must be included before Chebyshev.hpp
#include "../Spectral/Chebyshev_Spectral.hpp"
#include <boost/numeric/ublas/io.hpp>

// Assign our signal handlers
void (*SigHandleInt)(int sig);

bool abort_sim;

void Interrupt(int sig)
{
  // Re-assign the default signal handler again
  signal(SIGINT, SigHandleInt);
  abort_sim=true;
  std::cerr << "aborting... please wait for the current operations to finish" << std::endl;
}

void mSurf(DMatrix X, DMatrix Y, DArray u, DArray C)
{
  mxArray *surf[3];
  DMatrix x, y, ones;
  Map map;

  ones=boost::numeric::ublas::scalar_matrix<double> (X.size1(),X.size2(),1.);
  mexEvalString("clf;hold on");
  for( unsigned int j=0; j<C.size(); ++j ) {
	map=ArbitraryTriangleToStandardTriangle(C(j));
	x=map.T(0,0)*X+map.T(0,1)*Y+map.c(0)*ones;
	y=map.T(1,0)*X+map.T(1,1)*Y+map.c(1)*ones;
	array_conv(x,&surf[0]);
	array_conv(y,&surf[1]);
	array_conv(u(j),&surf[2]);
	mexCallMATLAB(0,NULL,3,surf,"surf");
  }
}

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )
{
  using namespace boost::numeric::ublas;
  int d, N, tmax;
  double dt;
  DVector theta, u0;
  IMatrix fudo;
  DMatrix x, y, X, Y, P, F, ones;
  DArray C, u, ux, uy, uxx, uyy, Du, Dux, Duy;
  Map map;
  Overlap overlap;

  if (nrhs < 4 || nrhs > 5)
	mexErrMsgTxt("Four or five input arguments required (N,d,dt,steps[,P]).");

  abort_sim=false;
  SigHandleInt = signal(SIGINT, Interrupt);

  N=(int)(*mxGetPr(prhs[0]));
  d=(int)(*mxGetPr(prhs[1]));
  dt=(double)(*mxGetPr(prhs[2]));
  tmax=(int)(*mxGetPr(prhs[3]));
  // decompose the domain into triangles (with clockwise orientation)
  C.resize(2*d*d);
  for( int j1=0; j1<d; ++j1 )
	for( int j2=0; j2<d; ++j2 ) {
	  C(2*(j1*d+j2)).resize(2,3);
	  C(2*(j1*d+j2)+1).resize(2,3);
	  C(2*(j1*d+j2))(0,0)=double(j2)/d;
	  C(2*(j1*d+j2))(0,1)=double(j2)/d;
	  C(2*(j1*d+j2))(0,2)=double(j2+1)/d;
	  C(2*(j1*d+j2))(1,0)=double(j1)/d;
	  C(2*(j1*d+j2))(1,1)=double(j1+1)/d;
	  C(2*(j1*d+j2))(1,2)=double(j1)/d;
	  C(2*(j1*d+j2)+1)(0,0)=double(j2+1)/d;
	  C(2*(j1*d+j2)+1)(0,1)=double(j2+1)/d;
	  C(2*(j1*d+j2)+1)(0,2)=double(j2)/d;
	  C(2*(j1*d+j2)+1)(1,0)=double(j1+1)/d;
	  C(2*(j1*d+j2)+1)(1,1)=double(j1)/d;
	  C(2*(j1*d+j2)+1)(1,2)=double(j1+1)/d;
	}
  overlap=Construct_Overlap(C,N);
  if( nrhs==5 )
	array_conv(&prhs[4],P);
  fudo=primalfudo(N);
  if( (nrhs != 5) || (P.size1()!=C.size()*fudo.size2())  ) {
	mexPrintf("Generating projection matrix...\n");
	P=Gen_Projection(C,N);
	mexPrintf("... done.\n");
  }
  F=Gen_Fixed(C,N);

  ones=boost::numeric::ublas::scalar_matrix<double> (N,N,1.);

  // initial conditions, NxN for each triangle
  u.resize(C.size());
  ux.resize(C.size());
  uy.resize(C.size());
  uxx.resize(C.size());
  uyy.resize(C.size());
  for( unsigned int j=0; j<C.size(); ++j ) {
	u(j).resize(N,N); u(j).clear();
	ux(j).resize(N,N); ux(j).clear();
	uy(j).resize(N,N); uy(j).clear();
	uxx(j).resize(N,N); uxx(j).clear();
	uyy(j).resize(N,N); uyy(j).clear();
  }
  theta.resize(N);
  for( int j=0; j<N; ++j )
	theta(j)=2.*M_PI*j/N;
  X.resize(N,N);
  Y.resize(N,N);
  for( int j1=0; j1<N; ++j1 )
	for( int j2=0; j2<N; ++j2 ) {
	  X(j1,j2)=(cos(theta(j1))+cos(theta(j2))+cos(theta(j1)-theta(j2)))/3.;
	  Y(j1,j2)=(sin(theta(j1))-sin(theta(j2))-sin(theta(j1)-theta(j2)))/3.;
	}
  // set up grids on the triangles and evaluate the function on each domain to get some initial data
  for( unsigned int j=0; j<C.size(); ++j ) {
	map=ArbitraryTriangleToStandardTriangle(C(j));
	x=map.T(0,0)*X+map.T(0,1)*Y+map.c(0)*ones;
	y=map.T(1,0)*X+map.T(1,1)*Y+map.c(1)*ones;
	for( int j1=0; j1<N; ++j1 )
	  for( int j2=0; j2<N; ++j2 ) {
		u(j)(j1,j2)=x(j1,j2)*x(j1,j2)*y(j1,j2)*y(j1,j2)+1.+exp(-10.*((x(j1,j2)-.5)*(x(j1,j2)-.5)+(y(j1,j2)-.5)*(y(j1,j2)-.5)));
	  }
  }
  // for( unsigned int j=0; j<C.size(); ++j )
	// u(j)=Filter(u(j));
  u0=prod(boost::numeric::ublas::identity_matrix<double> (F.size1(),F.size2())-F,AtoV(u,N,d));
  mexEvalString("fig=gcf;");
  mexEvalString("set(gcf,'DoubleBuffer','on');");
  mSurf(X,Y,u,C);
  mexEvalString("title('Initial conditions')");
  mexEvalString("axis([-.2 1.2 -.2 1.2])");
  mexEvalString("view([-10 50])");
  mexEvalString("drawnow");
  // mexEvalString("mov=avifile('movies/Heat_Equation_FE.avi','compression','none','fps',30);");
  // for( int t=0; t<15; ++t )
	// mexEvalString("mov = addframe(mov,getframe(gcf));");

  for( int t=1; t<=tmax; ++t ) {
	if( abort_sim )
	  break;
	for( unsigned int j=0; j<C.size(); ++j ) {
	  Du=TriGrad(u(j),C(j)); // gradient
	  ux(j)=Du(0);
	  uy(j)=Du(1);
	}
	// Smooth_Overlap(overlap,C,ux,1);
	// Smooth_Overlap(overlap,C,uy,1);
	// ux=VtoA(prod(P,AtoV(ux,N,d)),N,d);
	// uy=VtoA(prod(P,AtoV(uy,N,d)),N,d);
	for( unsigned int j=0; j<C.size(); ++j ) {
	  Dux=TriGrad(ux(j),C(j)); // uxx and uxy
	  Duy=TriGrad(uy(j),C(j)); // uyx and uyy
	  uxx(j)=Dux(0);
	  uyy(j)=Duy(1);
	}
	// uxx=VtoA(prod(P,AtoV(uxx,N,d)),N,d);
	// uyy=VtoA(prod(P,AtoV(uyy,N,d)),N,d);
	u+=dt*DArray(uxx+uyy); // apparently the compiler sees (uxx+uyy) as something other than a DArray... weird
	// u=VtoA(u0+prod(F,AtoV(u,N,d)),N,d);
	// Smooth_Overlap(overlap,C,u);
	u=VtoA(u0+prod(F,DVector(prod(P,AtoV(u,N,d)))),N,d);
	// for( unsigned int j=0; j<C.size(); ++j )
	  // u(j)=Filter(u(j));
	// if( ((t*dt<.3) && (t%int(1./500./dt)==0)) || ((t*dt>=.3)&&(t%int(1./100./dt)==0)) ) { // plot some graphs every now and then
	if( t%1==0 ) {
	  char str[80];
	  mxArray *vw;
	  mexCallMATLAB(1,&vw,0,NULL,"view");
	  mSurf(X,Y,u,C);
	  sprintf(str,"title('u at t=%3.2f (%3.1f%% dt=%g)')",t*dt,100.*t/tmax,dt);
	  mexEvalString(str);
	  mexEvalString("axis([-.2 1.2 -.2 1.2])");
	  mexCallMATLAB(0,NULL,1,&vw,"view");
	  mexEvalString("drawnow");
	  // mexEvalString("mov = addframe(mov,getframe(gcf));");
	}
  }
  // mexEvalString("mov = close(mov);");
  array_conv(P,&plhs[0]); // return P in case we want to reuse it
  array_conv(u,&plhs[1]);
  array_conv(overlap,&plhs[2],&plhs[3],&plhs[4],N);
  // Re-assign the default signal handler again
  signal(SIGINT, SigHandleInt);
}
