/* Create a spectral element method on the 2-torus using multivariate Chebyshev polynomials
 * Wave equation with forward Euler time-stepping.
 * To make a movie, uncomment the lines containing "mov"
 */
#include <iostream>
#include <cfloat>
#include <csignal>
#include <mex.h> // mex.h must be included before Chebyshev.hpp
#include "../Spectral/Chebyshev_Spectral.hpp"
#include <omp.h>
#include <boost/numeric/ublas/io.hpp>

// Assign our signal handlers
void (*SigHandleInt)(int sig);

bool abort_sim;

void Interrupt(int sig)
{
  // Re-assign the default signal handler again
  signal(SIGINT, SigHandleInt);
  abort_sim=true;
  std::cerr << "aborting... please wait for the current operations to finish" << std::endl;
}

void mSurf(DMatrix X, DMatrix Y, DArray u, DArray C)
{
  mxArray *surf[3];
  DMatrix x, y, ones;
  Map map;

  ones=boost::numeric::ublas::scalar_matrix<double> (X.size1(),X.size2(),1.);
  mexEvalString("clf;hold on");
  for( unsigned int j=0; j<C.size(); ++j ) {
	map=ArbitraryTriangleToStandardTriangle(C(j));
	x=map.T(0,0)*X+map.T(0,1)*Y+map.c(0)*ones;
	y=map.T(1,0)*X+map.T(1,1)*Y+map.c(1)*ones;
	array_conv(x,&surf[0]);
	array_conv(y,&surf[1]);
	array_conv(u(j),&surf[2]);
	mexCallMATLAB(0,NULL,3,surf,"surf");
  }
}

int FixBoundary(DArray *in, IMatrix e, DMatrix P)
{
  int N,d,iter=0;
  double err=0,lasterr=0;

  N=(*in)(0).size1();
  d=(int)sqrt((*in).size()/2);
  for( unsigned int j=0; j<(*in).size(); ++j )
	(*in)(j)=Filter((*in)(j));
  for( unsigned int j=0; j<e.size2(); ++j )
	err+=fabs((*in)(0)(e(0,j)/N,e(0,j)%N)-(*in)(1)(e(1,j)/N,e(1,j)%N));
  lasterr=err+1;
  while( err<lasterr ) {
	lasterr=err;
	(*in)=VtoA(prod(P,AtoV((*in),N,d)),N,d);
#pragma omp parallel for
	for( unsigned int j=0; j<(*in).size(); ++j )
	  (*in)(j)=Filter((*in)(j));
	err=0;
	for( unsigned int j=0; j<e.size2(); ++j )
	  err+=fabs((*in)(0)(e(0,j)/N,e(0,j)%N)-(*in)(1)(e(1,j)/N,e(1,j)%N));
	++iter;
  }
  return iter;
}

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )
{
  using namespace std;
  int d, N, tmax, iter;
  double dt;
  DVector theta;
  IMatrix fudo,e;
  DMatrix x, y, X, Y, P, ones;
  DArray C, u, v, ux, uy, uxx, uyy, Du, Dux, Duy;
  Map map;
  Overlap overlap;

  if (nrhs < 4 || nrhs > 5)
	mexErrMsgTxt("Four or five input arguments required (N,d,dt,steps[,P]).");

  abort_sim=false;
  SigHandleInt = signal(SIGINT, Interrupt);

  N=(int)(*mxGetPr(prhs[0]));
  d=(int)(*mxGetPr(prhs[1]));
  dt=(double)(*mxGetPr(prhs[2]));
  tmax=(int)(*mxGetPr(prhs[3]));
  // decompose the domain into triangles (with clockwise orientation)
  C.resize(2*d*d);
  for( int j1=0; j1<d; ++j1 )
	for( int j2=0; j2<d; ++j2 ) {
	  C(2*(j1*d+j2)).resize(2,3);
	  C(2*(j1*d+j2)+1).resize(2,3);
	  C(2*(j1*d+j2))(0,0)=double(j2)/d;
	  C(2*(j1*d+j2))(0,1)=double(j2)/d;
	  C(2*(j1*d+j2))(0,2)=double(j2+1)/d;
	  C(2*(j1*d+j2))(1,0)=double(j1)/d;
	  C(2*(j1*d+j2))(1,1)=double(j1+1)/d;
	  C(2*(j1*d+j2))(1,2)=double(j1)/d;
	  C(2*(j1*d+j2)+1)(0,0)=double(j2+1)/d;
	  C(2*(j1*d+j2)+1)(0,1)=double(j2+1)/d;
	  C(2*(j1*d+j2)+1)(0,2)=double(j2)/d;
	  C(2*(j1*d+j2)+1)(1,0)=double(j1+1)/d;
	  C(2*(j1*d+j2)+1)(1,1)=double(j1)/d;
	  C(2*(j1*d+j2)+1)(1,2)=double(j1+1)/d;
	}
  overlap=Construct_Overlap(C,N);
  e.resize(2,0);
  for( unsigned int j=0; j<overlap.edges.size(); ++j )
	if( overlap.edges(j).size()>1 ) {
	  e.resize(2,e.size2()+1,true);
	  e(0,e.size2()-1)=overlap.edges(j)(0)(1);
	  e(1,e.size2()-1)=overlap.edges(j)(1)(1);
	}
  for( unsigned int j=0; j<overlap.corners.size(); ++j )
	if( overlap.corners(j).size()>1 ) {
	  for( unsigned int k=1; k<overlap.corners(j)(0).size(); ++k ) {
		e.resize(2,e.size2()+1,true);
		e(0,e.size2()-1)=overlap.corners(j)(0)(k);
		e(1,e.size2()-1)=overlap.corners(j)(1)(k);
	  }
	}
  if( nrhs==5 )
	array_conv(&prhs[4],P);
  fudo=primalfudo(N);
  if( (nrhs != 5) || (P.size1()!=C.size()*fudo.size2())  ) {
	mexPrintf("Generating projection matrix...\n");
	P=Gen_Projection(C,N);
	mexPrintf("... done.\n");
  }

  ones=boost::numeric::ublas::scalar_matrix<double> (N,N,1.);

  // initial conditions, NxN for each triangle
  u.resize(C.size());
  v.resize(C.size());
  ux.resize(C.size());
  uy.resize(C.size());
  uxx.resize(C.size());
  uyy.resize(C.size());
  for( unsigned int j=0; j<C.size(); ++j ) {
	u(j).resize(N,N); u(j).clear();
	v(j).resize(N,N); v(j).clear();
	ux(j).resize(N,N); ux(j).clear();
	uy(j).resize(N,N); uy(j).clear();
	uxx(j).resize(N,N); uxx(j).clear();
	uyy(j).resize(N,N); uyy(j).clear();
  }
  theta.resize(N);
  for( int j=0; j<N; ++j )
	theta(j)=2.*M_PI*j/N;
  X.resize(N,N);
  Y.resize(N,N);
  for( int j1=0; j1<N; ++j1 )
	for( int j2=0; j2<N; ++j2 ) {
	  X(j1,j2)=(cos(theta(j1))+cos(theta(j2))+cos(theta(j1)-theta(j2)))/3.;
	  Y(j1,j2)=(sin(theta(j1))-sin(theta(j2))-sin(theta(j1)-theta(j2)))/3.;
	}
  // set up grids on the triangles and evaluate the function on each domain to get some initial data
  for( unsigned int j=0; j<C.size(); ++j ) {
	map=ArbitraryTriangleToStandardTriangle(C(j));
	x=map.T(0,0)*X+map.T(0,1)*Y+map.c(0)*ones;
	y=map.T(1,0)*X+map.T(1,1)*Y+map.c(1)*ones;
	for( int j1=0; j1<N; ++j1 )
	  for( int j2=0; j2<N; ++j2 ) {
		u(j)(j1,j2)=(exp(sin(4.*M_PI*x(j1,j2)-M_PI_2)+1.)-1.)*(exp(sin(2.*M_PI*y(j1,j2)-M_PI_2)+1.)-1.)/16.;
		// u(j)(j1,j2)=sin(4.*M_PI*x(j1,j2)-M_PI_2)*sin(2.*M_PI*y(j1,j2)-M_PI_2);
		v(j)(j1,j2)=0.;
	  }
  }
  u=VtoA(prod(P,AtoV(u,N,d)),N,d);
  v=VtoA(prod(P,AtoV(v,N,d)),N,d);
  // iter=FixBoundary(&u,e,P);
  // std::cout << "Iterations (u): " << iter << std::endl;
  // iter=FixBoundary(&v,e,P);
  // std::cout << "Iterations (v): " << iter << std::endl;
  mexEvalString("fig=gcf;");
  mexEvalString("set(gcf,'DoubleBuffer','on');");
  mSurf(X,Y,u,C);
  mexEvalString("title('Initial conditions')");
  mexEvalString("axis([-.2 1.2 -.2 1.2 -2 2])");
  mexEvalString("view([-10 50])");
  mexEvalString("drawnow");
  // mexEvalString("mov=avifile('movies/Wave_Equation_FE.avi','compression','none','fps',30);");
  // for( int t=0; t<15; ++t )
	// mexEvalString("mov = addframe(mov,getframe(gcf));");

  for( int t=1; t<=tmax; ++t ) {
	if( abort_sim )
	  break;
	for( int j=0; j<(int)C.size(); ++j ) {
	  Du=TriGrad(u(j),C(j)); // gradient
	  ux(j)=Du(0);
	  uy(j)=Du(1);
	}
	ux=VtoA(prod(P,AtoV(ux,N,d)),N,d);
	uy=VtoA(prod(P,AtoV(uy,N,d)),N,d);
	// iter=FixBoundary(&ux,e,P);
	// std::cout << "Iterations (ux): " << iter << std::endl;
	// iter=FixBoundary(&uy,e,P);
	// std::cout << "Iterations (uy): " << iter << std::endl;
	for( int j=0; j<(int)C.size(); ++j ) {
	  Dux=TriGrad(ux(j),C(j)); // uxx and uxy
	  Duy=TriGrad(uy(j),C(j)); // uyx and uyy
	  uxx(j)=Dux(0);
	  uyy(j)=Duy(1);
	}
	uxx=VtoA(prod(P,AtoV(uxx,N,d)),N,d);
	uyy=VtoA(prod(P,AtoV(uyy,N,d)),N,d);
	// iter=FixBoundary(&uxx,e,P);
	// std::cout << "Iterations (uxx): " << iter << std::endl;
	// iter=FixBoundary(&uyy,e,P);
	// std::cout << "Iterations (uyy): " << iter << std::endl;
	u+=dt*v;
	v+=dt*DArray(uxx+uyy); // apparently the compiler sees (uxx+uyy) as something other than a DArray
	u=VtoA(prod(P,AtoV(u,N,d)),N,d);
	v=VtoA(prod(P,AtoV(v,N,d)),N,d);
	// iter=FixBoundary(&u,e,P);
	// std::cout << "Iterations (u): " << iter << std::endl;
	// iter=FixBoundary(&v,e,P);
	// std::cout << "Iterations (v): " << iter << std::endl;
	if( t%int(1./500./dt)==0 ) { // plot some graphs every now and then
	  char str[80];
	  mxArray *vw;
	  mexCallMATLAB(1,&vw,0,NULL,"view");
	  mSurf(X,Y,u,C);
	  sprintf(str,"title('u at t=%3.2f (%3.1f%% dt=%g)')",t*dt,100.*t/tmax,dt);
	  mexEvalString(str);
	  mexEvalString("axis([-.2 1.2 -.2 1.2 -2 2])");
	  mexCallMATLAB(0,NULL,1,&vw,"view");
	  mexEvalString("drawnow");
	  // mexEvalString("mov = addframe(mov,getframe(gcf));");
	}
  }
  // mexEvalString("mov = close(mov);");
  // array_conv(P,&plhs[0]); // return P in case we want to reuse it
  array_conv(u,&plhs[0]);
  for( int j=0; j<e.size2(); ++j ) { // convert to MATLAB indices
	e(0,j)=(e(0,j)%N)*N+e(0,j)/N+1;
	e(1,j)=(e(1,j)%N)*N+e(1,j)/N+1;
  }
  array_conv(e,&plhs[1]);
  array_conv(ux,&plhs[2]);
  array_conv(uy,&plhs[3]);
  array_conv(uxx,&plhs[4]);
  array_conv(uyy,&plhs[5]);
  // Re-assign the default signal handler again
  signal(SIGINT, SigHandleInt);
}
