#include <iostream>
#include <cfloat>
#include <mex.h> // mex.h must be included before Chebyshev.hpp
#include "../Chebyshev/Chebyshev.hpp"
#include <boost/numeric/ublas/io.hpp>
#include <sys/time.h>

DMatrix func(int N)
{
  double x,y;
  DVector theta;
  DMatrix Values;

  Values.resize(N,N);
  theta.resize(N);
  for( int i=0; i<N; ++i )
	theta(i)=i*2.*M_PI/N;
  for( int j=0; j<N; ++j)
	for( int i=0; i<N; ++i) {
	  x = (cos(theta(i)) + cos(theta(j)) + cos(theta(i)-theta(j)))/3.;
	  y = (sin(theta(i)) - sin(theta(j)) - sin(theta(i)-theta(j)))/3.;
	  if( fabs(x) < DBL_EPSILON )
		x=0.;
	  if( fabs(y) < DBL_EPSILON )
		y=0.;

	  Values(i,j) = exp(sin(x)*sin(y));
	}

  return Values;
}

DMatrix gradx(int N)
{
  double x,y;
  DVector theta;
  DMatrix Values;

  Values.resize(N,N);
  theta.resize(N);
  for( int i=0; i<N; ++i )
	theta(i)=i*2.*M_PI/N;
  for( int j=0; j<N; ++j)
	for( int i=0; i<N; ++i) {
	  x = (cos(theta(i)) + cos(theta(j)) + cos(theta(i)-theta(j)))/3.;
	  y = (sin(theta(i)) - sin(theta(j)) - sin(theta(i)-theta(j)))/3.;
	  if( fabs(x) < DBL_EPSILON )
		x=0.;
	  if( fabs(y) < DBL_EPSILON )
		y=0.;

	  Values(i,j) = cos(x)*sin(y)*exp(sin(x)*sin(y));
	}

  return Values;
}

DMatrix grady(int N)
{
  double x,y;
  DVector theta;
  DMatrix Values;

  Values.resize(N,N);
  theta.resize(N);
  for( int i=0; i<N; ++i )
	theta(i)=i*2.*M_PI/N;
  for( int j=0; j<N; ++j)
	for( int i=0; i<N; ++i) {
	  x = (cos(theta(i)) + cos(theta(j)) + cos(theta(i)-theta(j)))/3.;
	  y = (sin(theta(i)) - sin(theta(j)) - sin(theta(i)-theta(j)))/3.;
	  if( fabs(x) < DBL_EPSILON )
		x=0.;
	  if( fabs(y) < DBL_EPSILON )
		y=0.;

	  Values(i,j) = sin(x)*cos(y)*exp(sin(x)*sin(y));
	}

  return Values;
}

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )
{
  using namespace std;
  int Nmax,Navg;
  double exact_int,exact_surface_area;
  DMatrix Values,dx,dy,S,I,T,IntT;
  DArray D;
  struct timeval t1,t2;

  if (nrhs != 2) {
	mexErrMsgTxt("Two input arguments required (Nmax,Navg).");
  } else if (nlhs > 2) {
	mexErrMsgTxt("Too many output arguments.");
  }

  Nmax=(int)(*mxGetPr(prhs[0]));
  Navg=(int)(*mxGetPr(prhs[1]));

  IntT = IntLookup_deltoid(Nmax);
  exact_int=CCInt(func(Nmax),IntT);
  S.resize(Nmax,Nmax);
  dx=gradx(Nmax);
  dy=grady(Nmax);
  for( int i=0; i<Nmax; ++i )
	for( int j=0; j<Nmax; ++j )
	  S(i,j)=sqrt(1.+dx(i,j)*dx(i,j)+dy(i,j)*dy(i,j));
  exact_surface_area=CCInt(S,IntT);

  I.resize(4,Nmax);
  I.clear();
  T.resize(3,Nmax);
  T.clear();
  mexPrintf("CCInt\n");
  for( int N=1; N<=Nmax; ++N ) {
	Values=func(N);
	T(0,N-1)=10.;
	for( int i=0; i<Navg; ++i ) {
	  gettimeofday(&t1,0);
	  I(0,N-1)=CCInt(Values,IntT);
	  gettimeofday(&t2,0);
	  T(0,N-1)=min(double(t2.tv_sec-t1.tv_sec)+double(t2.tv_usec-t1.tv_usec)*1e-6,T(0,N-1));
	}
	I(0,N-1)=fabs(I(0,N-1)-exact_int)/exact_int;
  }
  mexPrintf("Surface Area\n");
  for( int N=1; N<=Nmax; ++N ) {
	Values=func(N);
	T(1,N-1)=10.;
	for( int i=0; i<Navg; ++i ) {
	  gettimeofday(&t1,0);
	  I(1,N-1)=Surface_Area(Values,IntT);
	  gettimeofday(&t2,0);
	  T(1,N-1)=min(double(t2.tv_sec-t1.tv_sec)+double(t2.tv_usec-t1.tv_usec)*1e-6,T(1,N-1));
	}
	I(1,N-1)=fabs(I(1,N-1)-exact_surface_area)/exact_surface_area;
  }
  mexPrintf("Gradient\n");
  for( int N=1; N<=Nmax; ++N ) {
	Values=func(N);
	T(2,N-1)=10.;
	for( int i=0; i<Navg; ++i ) {
	  gettimeofday(&t1,0);
	  D=Gradient(Values);
	  gettimeofday(&t2,0);
	  T(2,N-1)=min(double(t2.tv_sec-t1.tv_sec)+double(t2.tv_usec-t1.tv_usec)*1e-6,T(2,N-1));
	}
	I(2,N-1)=norm_frobenius(D(0)-gradx(N))/norm_frobenius(gradx(N));
	I(3,N-1)=norm_frobenius(D(1)-grady(N))/norm_frobenius(grady(N));
  }
  array_conv(I,&plhs[0]);
  array_conv(T,&plhs[1]);
}
