#include <iostream>
#include <cfloat>
#include <mex.h> // mex.h must be included before Chebyshev.hpp
#include "../Chebyshev/Chebyshev.hpp"
#include <boost/numeric/ublas/io.hpp>
#include <sys/time.h>

DMatrix func(int N, DMatrix C)
{
  double x,y;
  DVector theta;
  DMatrix Values;
  Map map;

  map=ArbitraryTriangleToStandardTriangle(C);
  Values.resize(N,N);
  theta.resize(N);
  for( int i=0; i<N; ++i )
	theta(i)=i*2.*M_PI/N;
  for( int j=0; j<N; ++j)
	for( int i=0; i<N; ++i) {
	  double temp;
	  x = (cos(theta(i)) + cos(theta(j)) + cos(theta(i)-theta(j)))/3.;
	  y = (sin(theta(i)) - sin(theta(j)) - sin(theta(i)-theta(j)))/3.;
	  if( fabs(x) < DBL_EPSILON )
		x=0.;
	  if( fabs(y) < DBL_EPSILON )
		y=0.;
	  temp=map.T(1,0)*x+map.T(1,1)*y+map.c(1);
	  x=map.T(0,0)*x+map.T(0,1)*y+map.c(0);
	  y=temp;

	  Values(i,j) = exp(sin(x)*sin(y));
	  // Values(i,j) = x*x*x*y*y;
	}

  return Values;
}

DMatrix gradx(int N, DMatrix C)
{
  double x,y;
  DVector theta;
  DMatrix Values;
  Map map;

  map=ArbitraryTriangleToStandardTriangle(C);
  Values.resize(N,N);
  theta.resize(N);
  for( int i=0; i<N; ++i )
	theta(i)=i*2.*M_PI/N;
  for( int j=0; j<N; ++j)
	for( int i=0; i<N; ++i) {
	  double temp;
	  x = (cos(theta(i)) + cos(theta(j)) + cos(theta(i)-theta(j)))/3.;
	  y = (sin(theta(i)) - sin(theta(j)) - sin(theta(i)-theta(j)))/3.;
	  if( fabs(x) < DBL_EPSILON )
		x=0.;
	  if( fabs(y) < DBL_EPSILON )
		y=0.;
	  temp=map.T(1,0)*x+map.T(1,1)*y+map.c(1);
	  x=map.T(0,0)*x+map.T(0,1)*y+map.c(0);
	  y=temp;

	  Values(i,j) = cos(x)*sin(y)*exp(sin(x)*sin(y));
	  // Values(i,j) = 3*x*x*y*y;
	}

  return Values;
}

DMatrix grady(int N, DMatrix C)
{
  double x,y;
  DVector theta;
  DMatrix Values;
  Map map;

  map=ArbitraryTriangleToStandardTriangle(C);
  Values.resize(N,N);
  theta.resize(N);
  for( int i=0; i<N; ++i )
	theta(i)=i*2.*M_PI/N;
  for( int j=0; j<N; ++j)
	for( int i=0; i<N; ++i) {
	  double temp;
	  x = (cos(theta(i)) + cos(theta(j)) + cos(theta(i)-theta(j)))/3.;
	  y = (sin(theta(i)) - sin(theta(j)) - sin(theta(i)-theta(j)))/3.;
	  if( fabs(x) < DBL_EPSILON )
		x=0.;
	  if( fabs(y) < DBL_EPSILON )
		y=0.;
	  temp=map.T(1,0)*x+map.T(1,1)*y+map.c(1);
	  x=map.T(0,0)*x+map.T(0,1)*y+map.c(0);
	  y=temp;

	  Values(i,j) = sin(x)*cos(y)*exp(sin(x)*sin(y));
	  // Values(i,j) = 2*x*x*x*y;
	}

  return Values;
}

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )
{
  using namespace std;
  int Nmax,Navg;
  double exact_int,exact_surface_area;
  DMatrix Values,dx,dy,S,I,T,IntT,C;
  DArray D;
  struct timeval t1,t2;

  if (nrhs != 2) {
	mexErrMsgTxt("Two input arguments required (Nmax,Navg).");
  } else if (nlhs > 2) {
	mexErrMsgTxt("Too many output arguments.");
  }

  C.resize(2,3);
  C(0,0)=0.;C(0,1)=0.;C(0,2)=1.;
  C(1,0)=0.;C(1,1)=1.;C(1,2)=0.;

  Nmax=(int)(*mxGetPr(prhs[0]));
  Navg=(int)(*mxGetPr(prhs[1]));

  IntT = IntLookup_triangle(Nmax);
  exact_int=TriCCInt(func(Nmax,C),C,IntT);
  S.resize(Nmax,Nmax);
  dx=gradx(Nmax,C);
  dy=grady(Nmax,C);
  for( int i=0; i<Nmax; ++i )
	for( int j=0; j<Nmax; ++j )
	  S(i,j)=sqrt(1.+dx(i,j)*dx(i,j)+dy(i,j)*dy(i,j));
  exact_surface_area=TriCCInt(S,C,IntT);

  I.resize(4,Nmax);
  I.clear();
  T.resize(3,Nmax);
  T.clear();
  for( int N=1; N<=Nmax; ++N ) {
	Values=func(N,C);
	T(0,N-1)=10.;
	for( int j=0; j<Navg; ++j ) {
	  gettimeofday(&t1,0);
	  I(0,N-1)=TriCCInt(Values,C,IntT);
	  gettimeofday(&t2,0);
	  T(0,N-1)=min(T(0,N-1),double(t2.tv_sec-t1.tv_sec)+double(t2.tv_usec-t1.tv_usec)*1e-6);
	}
	I(0,N-1)=fabs(I(0,N-1)-exact_int)/exact_int;
  }
  for( int N=1; N<=Nmax; ++N ) {
	Values=func(N,C);
	T(1,N-1)=10.;
	for( int j=0; j<Navg; ++j ) {
	  gettimeofday(&t1,0);
	  I(1,N-1)=TriSurf(Values,C,IntT);
	  gettimeofday(&t2,0);
	  T(1,N-1)=min(T(1,N-1),double(t2.tv_sec-t1.tv_sec)+double(t2.tv_usec-t1.tv_usec)*1e-6);
	}
	I(1,N-1)=fabs(I(1,N-1)-exact_surface_area)/exact_surface_area;
  }
  for( int N=1; N<=Nmax; ++N ) {
	Values=func(N,C);
	T(2,N-1)=10.;
	for( int j=0; j<Navg; ++j ) {
	  gettimeofday(&t1,0);
	  D=TriGrad(Values,C);
	  gettimeofday(&t2,0);
	  T(2,N-1)=min(T(2,N-1),double(t2.tv_sec-t1.tv_sec)+double(t2.tv_usec-t1.tv_usec)*1e-6);
	}
	I(2,N-1)=norm_frobenius(D(0)-gradx(N,C))/norm_frobenius(gradx(N,C));
	I(3,N-1)=norm_frobenius(D(1)-grady(N,C))/norm_frobenius(grady(N,C));
  }
  array_conv(I,&plhs[0]);
  array_conv(T,&plhs[1]);
}
