function [I] = CCInt(Function,N,IntT)
% [I] = CCInt(Function,N,IntT)
% Clenshaw-Curtis integration over the deltoid using precomputed integrals of Chebyshev polynomials.
%
% Input:
%	Function (string, function, inline function, N*N double) = function to integrate (don't use symbolic variables!)
%	N (int) = downscaling factor
%	IntT (N*N double) = table of integrals of Chebyshev polynomials from IntLookup_triangle() or IntLookup_deltoid()
%
% Output:
%	I (int) = the integral over the inscribed triangle.

% Brett Ryland, Nov 2008

% construct the real coordinates at the Chebyshev extremal points (x,y)=(real(z),imag(z)).
theta = [0:N-1]*2*pi/N;
x = 1/3*(exp(i*theta.'*ones(1,N)) + exp(-i*ones(N,1)*theta) + exp(-i*(theta.'*ones(1,N)-ones(N,1)*theta)));
y=imag(x);
x=real(x);

% figure out what has been input
if ischar(Function)
  if exist(Function) == 2 % 'function'
	Values = feval(Function,x,y);
  else % 'string'
	Values = eval(Function).*ones(size(x));
  end
else
  if isnumeric(Function) % 'numeric'
	if size(Function,1) == N && size(Function,2) == N
	  Values = Function;
	else
	  disp('Input array is the wrong size.')
	  Derivative = 0;
	  return
	end
  elseif exist(Function) % 'inline'
	Values = feval(Function,x,y);
  end
end

% perform a FFT on symmetrised Values to obtain the Chebyshev coefficients
% they are not real but the imaginary part vanishes due to the symmetries of the integral below, so we only take the real part here
% e.g., check Coefficients(find(orbnum==j)) for j=1:Ndfudo, the positive and negative imaginary components cancel
Coefficients = real(fft2(Values)/N^2);

% obtain the orbit information of the multivariate Chebyshev polynomials in the fundamental domain.
g = dualfudo(N);
Ndfudo = size(g,2);
G=Gdual;
Ndual = size(G,3);
[orbnum,orbact,iso] = orbit_info(G,g,N);
Niso = sum(iso~=0);

% perform the integration
I = sum(Coefficients(g(1,:)+1+N*g(2,:)).*IntT(g(1,:)+1+size(IntT,1)*g(2,:))*Ndual./Niso);
