function [overlap,edges,corners] = Construct_Overlap(C,N)
% [overlap,edges,corners] = Construct_Overlap(C,N)
% Construct the overlap information from the triangles and downscaling.
%
% Input:
%	C (2x3xd) = corners of the triangles
%	N (int) = downscaling
%
% Output:
%	overlap {dxdx2} = overlap information
%		{j1,j2,1}(2xn) = coordinates of points from deltoid j2 in triangle j1 (when j2=j1, this only contains the points that are in the triangle due to wrapping)
%		{j1,j2,2}(1xn) = linear index into data of {j1,j2,1}
%	edges {#edgepoints x 1} = edge overlap information
%		{e}(:,:) = [triangle indices] of coinciding points on edges of two triangles (each {e} should be 2x2)
%	corners {#corners x 1} = corner overlap information
%		{c}(:,:) = [triangle indices] of coinciding points on corners of multiple triangles

tol=1e-10; % tolerance for detecting coinciding points
% wrapping function, adjust for desired geometry.
wrap=inline('mod(x+1e-10,1)-1e-10'); % periodic unit square
% wrap=inline('x'); % unit square without periodicity

d=size(C,3);
theta = [0:N-1]*2*pi/N;
x = reshape(1/3*(exp(i*theta.'*ones(1,N)) + exp(-i*ones(N,1)*theta) + exp(-i*(theta.'*ones(1,N)-ones(N,1)*theta))),1,N^2);
Y=imag(x); X=real(x);
% overlap: first two indices are which deltoid or triangle, third index is coordinates in triangle or grid point in deltoid
overlap=cell(d,d,2);
% indices to points inside your own triangle are the same for all triangles
[T,centroid]=ArbitraryTriangleToStandardTriangle(C(:,:,1));
x=T(1,:)*[X;Y]+centroid(1);
y=T(2,:)*[X;Y]+centroid(2);
local_points=triangle_interior_points([x;y],C(:,:,1));
edge_points=triangle_interior_points([x;y],C(:,:,1),[0 1 0]); % N-3 with 6-fold symmetry and 3 with 3-fold symmetry (midpoints of edges)
corner_points=triangle_interior_points([x;y],C(:,:,1),[0 0 1]); % 6-fold symmetry
for j1=1:d
  for j2=1:d
	[T,centroid]=ArbitraryTriangleToStandardTriangle(C(:,:,j2));
	x=T(1,:)*[X;Y]+centroid(1);
	y=T(2,:)*[X;Y]+centroid(2);
	% find points from other deltoids that are in this triangle
	overlap{j1,j2,2}=setdiff(triangle_interior_points(wrap([x;y]),C(:,:,j1)),local_points); % index of points in deltoid
	% overlap{j1,j2,2}=triangle_interior_points(wrap([x;y]),C(:,:,j1)); % index of points in deltoid
	% get their coordinates
	overlap{j1,j2,1}=wrap([x(overlap{j1,j2,2});y(overlap{j1,j2,2})]); % coords in triangle
  end
end

% find coinciding corner points
if ~isempty(corner_points)
  cnrs=uniq(wrap(reshape(C,2,3*d).'),tol); % unique list of the corners of all triangles
  corners=cell(size(cnrs,1),1);
  for j1=1:size(cnrs,1)
	corners{j1}=[];
	for j2=1:d
	  [T,centroid]=ArbitraryTriangleToStandardTriangle(C(:,:,j2));
	  x=T(1,:)*[X;Y]+centroid(1);
	  y=T(2,:)*[X;Y]+centroid(2);
	  [c,ci,cj]=uniq([x(corner_points);y(corner_points)].',tol); % sorted unique corner points with tolerance tol
	  I=find(max(abs(wrap(c-cnrs(j1*ones(1,size(c,1)),:))),[],2)<tol);
	  if ~isempty(I)
		corners{j1}=[corners{j1};j2 corner_points(ci(I))];
	  end
	end
  end
else
  corners=cell(0,1);
end

% find coinciding edge points
edpts=[];
for j=1:d
  [T,centroid]=ArbitraryTriangleToStandardTriangle(C(:,:,j));
  x=T(1,:)*[X;Y]+centroid(1);
  y=T(2,:)*[X;Y]+centroid(2);
  edpts=[edpts wrap([x(edge_points);y(edge_points)])];
end
edpts=uniq(edpts.',tol); % unique list of the edge points of all triangles
edges=cell(size(edpts,1),1);
for j1=1:size(edpts,1)
  edges{j1}=[];
  for j2=1:d
	[T,centroid]=ArbitraryTriangleToStandardTriangle(C(:,:,j2));
	x=T(1,:)*[X;Y]+centroid(1);
	y=T(2,:)*[X;Y]+centroid(2);
	[e,ei,ej]=uniq([x(edge_points);y(edge_points)].',tol); % sorted unique edge points with tolerance tol
	I=find(max(abs(wrap(e-edpts(j1*ones(1,size(e,1)),:))),[],2)<tol);
	if ~isempty(I)
	  edges{j1}=[edges{j1};j2 edge_points(ei(I))];
	end
  end
end
