function [Derivative] = Gradient(Function,N)
% [Derivative] = Gradient(Function,N)
% Compute the gradient of the Chebyshev polynomial at all the points in the downscaled lattice.
% The gradient is computed w.r.t. the real (x1,x2) variables.
% The general process is:
%	- Find the coefficients a(k) by cosine FFT (by extending domain symmetrically).
%	- Modify the a(k) so that the FFT is now the derivative via the recursion formula.
%	- Apply an IFFT to recover the derivative at the internal points.
% For polynomials of degree n, the gradient is exact if N>2n.
%
% Input:
%	Function 'string', function, variable or inline = name of function that is being approximated by the Chebyshev polynomial.
%		Make sure you use elementwise operators in the function, or expect errors.
%		If you input a variable, the values should correspond to some function evaluated at the Chebyshev extremal points.
%	N (int) = root lattice downscaling factor
%
% Output:
%	Derivative (N, N, 2) = the gradient at each downscaled lattice point.

% Brett Ryland, Dec 2008

% Construct the (x1,x2) coordinates
theta = [0:N-1]*2*pi/N;
x = 1/3*(exp(i*theta.'*ones(1,N)) + exp(-i*ones(N,1)*theta) + exp(-i*(theta.'*ones(1,N)-ones(N,1)*theta)));
y=imag(x); % switch to real and imaginary coordinates
x=real(x);

% Figure out what has been input
if ischar(Function)
  if exist(Function) == 2 % 'function'
	Values = feval(Function,x,y);
  else % 'string'
	Values = eval(Function);
  end
else
  if isnumeric(Function) % 'numeric'
	if size(Function,1) == N && size(Function,2) == N
	  Values = Function;
	else
	  disp('Input array is the wrong size.')
	  Derivative = 0;
	  return
	end
  elseif exist(Function) % 'inline'
	Values = feval(Function,x,y);
  end
end

% get the sorted dual fundamental domain
% note: the outermost entries of g could also be thrown away as b is zero for those g, but the computational gain is marginal
g = dualfudo_sorted(N);

% perform a 2D FFT on Values to obtain the Chebyshev coefficients
a = fft2(Values);

% pseudo Fourier coefficients, A is a inside the dual fundamental domain and zero outside
A=zeros(max(g(1,:))+2);
for j=1:size(g,2)
	A(g(1,j)+1,g(2,j)+1)=a(g(1,j)+1,g(2,j)+1);
	A(g(2,j)+1,g(1,j)+1)=a(g(2,j)+1,g(1,j)+1);
end

% modify the Fourier coefficients via the recursion formula
% |W|*k*A(k) = \sum_{l=1}^{2} \sum_{g\in W} b(k-g^{T}e_{l})_{l}*(g^{T}e_{l})
if N<5
  b = zeros(N+2,N+2,2); % make b slightly larger to allow the recursion formula to work
else
  b = zeros(N,N,2);
end
for j=size(g,2):-1:1
  if g(2,j)>0
	b(g(1,j)+1,g(2,j)+1,1)=3*(g(1,j)+1)*A(g(1,j)+2,g(2,j)+1)+b(g(1,j)+3,g(2,j),1)+b(g(1,j)+3,g(2,j)+1,2)-b(g(1,j)+1,g(2,j)+2,2);
  else % lower edge, b(m+2,0)=b(m+1,2)
	b(g(1,j)+1,1,1)=3*(g(1,j)+1)*A(g(1,j)+2,1)+b(g(1,j)+2,2,1)+b(g(1,j)+3,1,2)-b(g(1,j)+1,2,2);
  end
  if g(1,j)>0
	b(g(1,j)+1,g(2,j)+1,2)=3*(g(2,j)+1)*A(g(1,j)+1,g(2,j)+2)+b(g(1,j),g(2,j)+3,2)+b(g(1,j)+1,g(2,j)+3,1)-b(g(1,j)+2,g(2,j)+1,1);
  else % origin
	b(1,1,2)=3*A(1,2)+b(2,2,2)+b(1,3,1)-b(2,1,1);
  end
  % symmetry b(m,n,[1;2])=conj(b(n,m,[2;1])) from a(m,n)=conj(a(n,m)) and symmetry of recursion formulae
  b(g(2,j)+1,g(1,j)+1,[1;2])=conj(b(g(1,j)+1,g(2,j)+1,[2;1]));
end
if N<5
  b = b(1:N,1:N,:); % truncate b to appropriate size
end

% apply symmetries
for j=1:size(g,2)
  b(mod(-g(1,j),N)+1,mod(g(1,j)+g(2,j),N)+1,:)=b(g(1,j)+1,g(2,j)+1,:);
  b(mod(g(1,j)+g(2,j),N)+1,mod(-g(2,j),N)+1,:)=b(g(1,j)+1,g(2,j)+1,:);
  b(mod(-g(1,j)-g(2,j),N)+1,g(1,j)+1,:)=b(g(1,j)+1,g(2,j)+1,:);
  b(g(2,j)+1,mod(-g(1,j)-g(2,j),N)+1,:)=b(g(1,j)+1,g(2,j)+1,:);
  b(mod(-g(2,j),N)+1,mod(-g(1,j),N)+1,:)=b(g(1,j)+1,g(2,j)+1,:);
  b(mod(-g(2,j),N)+1,mod(g(2,j)+g(1,j),N)+1,:)=b(g(2,j)+1,g(1,j)+1,:);
  b(mod(g(2,j)+g(1,j),N)+1,mod(-g(1,j),N)+1,:)=b(g(2,j)+1,g(1,j)+1,:);
  b(mod(-g(2,j)-g(1,j),N)+1,g(2,j)+1,:)=b(g(2,j)+1,g(1,j)+1,:);
  b(g(1,j)+1,mod(-g(2,j)-g(1,j),N)+1,:)=b(g(2,j)+1,g(1,j)+1,:);
  b(mod(-g(1,j),N)+1,mod(-g(2,j),N)+1,:)=b(g(2,j)+1,g(1,j)+1,:);
end

% take the 2D IFFT to recover the gradient
Derivative(:,:,1) = ifft2(b(:,:,1)); % in the x direction
Derivative(:,:,2) = ifft2(b(:,:,2)); % in the y direction

% multiply by J_x(z)^{-T} to get the derivative in x coordinates
J=[1 1;i -i];
Derivative=reshape((J*reshape(Derivative,N*N,2).').',N,N,2);
