function [IntT] = IntLookup_triangle(N)
% [IntT] = IntLookup_triangle(N)
% create a table of Integration values over the triangle for all multivariate Chebyshev polynomials
% within the dual fundamental domain of the triangle for a given value of the downscaling lattice.
%
% Input:
%	N (1x1 double) = lattice downscaling factor
%
% Output:
%	IntT (max(k1) x max(k2) double) = integral over the triangle of each Chebyshev polynomial in the fundamental domain
%		note that only such Chebyshev polynomials that appear get evaluated so matrix is sparse

% figure out which Chebyshev polynomials we need
g = dualfudo(N);

% construct the integral functions
% k1~=0, k2~=0, k1+k2~=0
f1=inline('1/k1/k2*(exp(i*k1*pi/3)*(exp(i*k2*2*pi/3)-exp(i*k2*pi/3))-exp(i*k1*pi)*(exp(i*k2*pi)-exp(i*k2*2*pi/3)))+1/k1/(k2+k1)*(exp(2/3*i*pi*k1)*(exp(i*pi*k2)+exp(1/3*i*pi*k2))-exp(2/3*i*pi*k2)*(exp(i*k1*pi)+exp(1/3*i*k1*pi)))','k1','k2');
% k1==0, k2~=0 or k1~=0, k2==0
% f2=inline('i*pi/3/k*(exp(i*k*pi/3)-exp(i*k*pi))+1/k^2*(2*exp(i*k*2*pi/3)-exp(i*k*pi)-exp(i*k*pi/3))','k'); % not actually needed
% k1+k2==0
% f3=inline('2/k1^2*(1-cos(pi/3*k1))+2*pi/3/k1*sin(pi/3*k1)','k1'); % not actually needed
% k1==0, k2==0 => f=pi^2/3

% construct the Weyl group and Jacobian determinant modifiers to k
W=Weyl(Cartan('A',2));
l=[1 -1 1 -1 -2 2;1 -1 -2 2 1 -1;1 -1 1 -1 1 -1];

IntT=zeros(max(g(1,:))+1,max(g(2,:))+1);
for n=1:size(g,2)
  k=g(:,n);
  if mod(k(1)-k(2),3) ~= 0
	continue; % integral over the triangle is zero unless mod(k(1)-k(2),3) == 0
  end

  % construct the points to evaluate the integral at
  K=[];
  for j=1:6
	for j2=1:6
	  K=[K [W(:,:,j)*k+l(1:2,j2); l(3,j2)]];
	end
  end
  % sum over the integrals
  for j=1:36
	% the first four possibilities always occur in +- pairs which cancel, so ignore them.
	if K(1,j)==0 & K(2,j)==0
	  % IntT(g(1,n)+1,g(2,n)+1)=IntT(g(1,n)+1,g(2,n)+1)+K(3,j)*pi^2/3;
	  continue;
	end
	if K(1,j)==0 % K(2,j)~=0 if we got to here
	  % IntT(g(1,n)+1,g(2,n)+1)=IntT(g(1,n)+1,g(2,n)+1)+K(3,j)*f2(K(2,j));
	  continue;
	end
	if K(2,j)==0 % K(1,j)~=0 if we got to here
	  % IntT(g(1,n)+1,g(2,n)+1)=IntT(g(1,n)+1,g(2,n)+1)+K(3,j)*f2(K(1,j));
	  continue;
	end
	if K(1,j)+K(2,j)==0 % K(1,j)~=0, K(2,j)~=0 if we got to here
	  % IntT(g(1,n)+1,g(2,n)+1)=IntT(g(1,n)+1,g(2,n)+1)+K(3,j)*f3(K(1,j));
	  continue;
	end
	IntT(g(1,n)+1,g(2,n)+1)=IntT(g(1,n)+1,g(2,n)+1)+K(3,j)*f1(K(1,j),K(2,j));
  end
end
% remove imaginary round-off error
% i/2 from |J_x(z)|, -1/9 from |J_z(theta)|, 1/6 from 1/|W| and - from clockwise to anti-clockwise orientation of triangle in x and theta domains
IntT=real(IntT*i/9/6/2);
