function [out] = Smooth_Overlap(overlap,edges,corners,C,in,iters)
% [out] = Smooth_Overlap(overlap,edges,corners,C,in,iters)
% On a triangular decomposition of a domain the deltoids overlap, thus some points in each deltoid are
% some other triangle(s). These points can be made to agree with the data from those triangles by
% interpolating these points on the triangles.
%
% Input:
%	overlap {nxnx2}(2xm,1xm) = index of which points belong to which deltoid and are in which triangle
%		n = number of triangles, m = number of points from a deltoid in that triangle
%	edges {ex1}(mx2) = [triangle,index] pairs that contain the point e which lies on the edge of two triangles
%	corners {cx1}(mx2) = [triangle,index] pairs that contain the point c which lies at the corner of several triangles
%	C (2x3xn) = triangle corners
%	in (NxNxn) = values of the function to be smoothed
%		N = downscaling of root lattice
%	iters [optional] = number of iterations, set to 0 or don't specify to iterate until convergence
%
% Output:
%	out (NxNxn) = smooth values of the function
%
% Brett Ryland, Feb 2009

N=size(in,1);
n=size(in,3);
out=in;

if nargin < 6
  iters=0;
end

W=Weyl(Cartan('A',2));
% average the corners
for j=1:size(corners,1)
  avg=0;
  for j2=1:size(corners{j},1)
	for j3=2:size(corners{j},2)
	  k=[mod(corners{j}(j2,j3)-1,N);floor((corners{j}(j2,j3)-1)/N)];
	  avg=avg+in(k(1)+1,k(2)+1,corners{j}(j2,1));
	end
  end
  avg=avg/j2/(j3-1);
  for j2=1:size(corners{j},1)
	for j3=2:size(corners{j},2)
	  k=[mod(corners{j}(j2,j3)-1,N);floor((corners{j}(j2,j3)-1)/N)];
	  for j4=1:size(W,3)
		out(mod(W(:,1,j4).'*k,N)+1,mod(W(:,2,j4).'*k,N)+1,corners{j}(j2,1))=avg;
	  end
	end
  end
end

% average the edges
for j=1:size(edges,1)
  avg=0;
  for j2=1:size(edges{j},1)
	k=[mod(edges{j}(j2,2)-1,N);floor(edges{j}(j2,2)/N)];
	avg=avg+in(k(1)+1,k(2)+1,edges{j}(j2,1));
  end
  avg=avg/j2;
  for j2=1:size(edges{j},1)
	k=[mod(edges{j}(j2,2)-1,N);floor(edges{j}(j2,2)/N)];
	for j3=1:size(W,3)
	  out(mod(W(:,1,j3).'*k,N)+1,mod(W(:,2,j3).'*k,N)+1,edges{j}(j2,1))=avg;
	end
  end
end

err=0;err_last=0;
iteration=0;
while iteration<2 || err<err_last
  iteration=iteration+1;
  in=out;
  % for each triangle (j1) and deltoid (j2), interpolate all the values that lie inside the triangle from the deltoid
  for j1=1:n
	oli=[];
	for j2=1:n
	  oli=[oli (j2-1)*N^2+overlap{j1,j2,2}];
	end
	olv=cat(2,overlap{j1,:,1});
	out(oli)=Interp(olv,in(:,:,j1),C(:,:,j1));
  end
  err_last=err;
  err=max(max(max(abs(out-in))));
  if max(max(max(abs(out))))<1e-20 % in case the solution is 0 and continues converging forever...
	break
  end
  if (iters ~= 0) && (iteration >= iters)
	break
  end
end
