function [Area,Derivative] = Surface_Area(Function,N,IntT)
% [Area,Derivative] = Surface_Area(Function,N,IntT)
% Calculate the surface area of a triangular surface patch using Clenshaw-Curtis
% quadrature on the derivative of a function on deltoid.
%
% Input:
%	Function 'string', function, variable or inline = name of function that is being approximated by the Chebyshev polynomial.
%		Make sure you use elementwise operators in the function, or expect errors.
%		If you input a variable, the values should correspond to some function evaluated at the Chebyshev extremal points.
%	N (int) = root lattice downscaling factor
%
% Output:
%	Area (double) = the surface area of the function defined on the Deltoid.
%	Derivative (NxN double) = the gradient of the function at the Chebyshev extremal points.

% Brett Ryland, Mar 2008

% Calculate the gradient of Function at the Chebyshev extremal points.
Derivative = Gradient(Function,N);

% Construct the function to be integrated.
% Area element dA = |cross(dfx,dfy)|, where dfx = [dx;0;f(dx,0)-f(0,0)], dfy = [0;dy;f(0,dy)-f(0,0)]
% taking limit dx->0, dy->0 gives A = int(sqrt(1+(df/dx)^2+(df/dy)^2))dxdy.
Integrand = sqrt(1+Derivative(:,:,1).^2+Derivative(:,:,2).^2);

% Integrate the function over the domain.
Area = CCInt(Integrand,N,IntT);
