function [orbitnumber,orbitaction,isotropy] = orbit_info(G,fudo,N)
% Information on orbits of matrix group acting on a periodic lattice
%
% IN:
%   G(2,2,grpsize) : matrix group acting on 2D lattice. We require that G(:,:,1) is the identity.
%   fudo(2,nfudo) : selection of points in fundamental domain of G (no correctness check done here!)
%      fudo(:,j) returns the two coordinates of point j in fudo
%   N : periodicity of lattice (assumed equal in both directions)
%
% OUT: 
%   orbitnumber(N,N) :  for each lattice point k = [k1;k2], 
%      orbitnumber(k1+1,k2+1) returns the number of the corresponding point in fudo
%
%   orbitaction(N,N) :  for each lattice point k = [k1;k2], 
%      orbitaction(k1+1,k2+1) returns the number of _an_ element of G, such that
%      this element applied to the fudo point of the orbit yields k. (Defined up to isotropy).
%      orbitaction(k1+1,k2+1) == 1 iff [k1;k2] \in fudo
%
%   isotropy(grpsize,nfudo) : for each point j in fudo, isotropy(:,j) returns the
%      isotropy subgroup of G at this point. The number 0 is used as fill_in to get 
%      square matrix. Thus, to find isotropy subgroup at a given fudo point j, do:
%      iso  = nonzeros(isotropy(:,j))
%

grpsize = size(G,3);
nfudo = size(fudo,2);
orbitnumber = zeros(N,N);
orbitaction = zeros(N,N);
isotropy = zeros(grpsize,nfudo);
isocnt = zeros(1,nfudo); % count (growing) list of isotropy subgroup
for igr = 1:grpsize
	gk = mod(G(:,:,igr)*fudo,N); %action of group element on fudo points
	fixpts = find(sum(mod(gk-fudo,N))==0);
	isocnt(fixpts) = isocnt(fixpts)+1;
	for j = fixpts
		isotropy(isocnt(j),j) = igr;
	end
	for j = 1:nfudo
		k1 = gk(1,j)+1; k2 = gk(2,j)+1;
		orbitnumber(k1,k2) = j;
		orbitaction(k1,k2) = igr;
	end
end
% for points in fudo, we want orbit action to return 1
for j = 1:nfudo
		k1 = fudo(1,j)+1; k2 = fudo(2,j)+1;
		orbitaction(k1,k2) = 1;
end
