function [ndx] = triangle_interior_points(points, C, flags)
% [ndx] = triangle_interior_points(points, C, flags)
% return an index to the points that are interior to the triangle C
% computed by projecting the test point onto the normals of the edge vectors
% and testing if the magnitudes all have the same sign
% http://www.scribd.com/doc/2941109/Constructing-A-PointInTriangle-Matrix
%
% Input:
%	points (2xN) = list of points to test
%	C (2x3) = corners of the triangle
%	flags (1x4 bool) = [interior edge corner outer_edge] indicates which set of points to include
%
% Output:
%	ndx (nx1) = index of points inside the triangle

if nargin<3
  flags=[1 1 1 1]; % if not specified, return everything
end

% general 3D implementation
% A=[C.' zeros(3,1)];
% B=[-1 1 0;0 -1 1;1 0 -1]*A;
% n=cross(B(1,:),B(2,:));
% D=B*[0 -n(3) n(1);n(3) 0 -n(1);-n(2) n(1) 0];
% G=[D -dot(A,D,2)];
% ndx=find(abs(sum(sign(G*[points;zeros(1,size(points,2));ones(1,size(points,2))])))==3);

% optimised 2D implementation
B=circshift(C.',2)-C.';
n=B(1,1)*B(2,2)-B(1,2)*B(2,1);
D=n*[B(:,2) -B(:,1)];
G=[D -dot(C.',D,2)];
sgn=G*[points;ones(1,size(points,2))];
sgn(abs(sgn)<1e-13)=0; % tolerance to being on an edge
ndx=[];
m=length(flags);
if flags(1)
  ndx=find(abs(sum(sign(sgn)))==3); % ==3 for interior
end
if m>1 && flags(2)
  ndx=[ndx find(abs(sum(sign(sgn)))==2)]; % ==2 for edges
end
if m>2 && flags(3)
  ndx=[ndx find(sum(abs(sign(sgn)))==1)]; % corners (note that abs and sum are swapped)
end
if m>3 && flags(4)
  ndx=[ndx find(abs(sum(sign(sgn)))==0)]; % ==0 for external edges
end
